// java

package edu.washington.cac.calendar.data;

import java.sql.SQLException;
import java.util.Iterator;

import edu.washington.cac.calendar.db.Caldata;
import edu.washington.cac.myuw.servlet.session.UWSession;

/**
  An interface that can be used for an object that can either be a Cache
  or something that acts like it.
  @author Greg Barnes
  @version 1.0
 */
/* So far, this is only used indirectly, to make EventsI work */
public interface CacheI
{
  /**
    @return The number of items in the Cache
   */
  public int size();

  /**
    @return All the items in the cache
   */
  public Iterator elements();

  /**
    Add an item to the cache
    @param c Item to add
    @exception SQLException if there is a database problem
   */
  public void add(Cacheable c) throws SQLException;

  /**
    Add an item to the cache
    @param c Item to add
    @param saveToDB Save to database as well?
   */
  public void add(Cacheable c, boolean saveToDB) throws SQLException;

  /**
    delete an item from the Cache 

    @param c the item to delete
    @exception ItemException if there's a problem with c 
    @exception SQLException if there is a problem with the database
   */
  public void delete(Cacheable c) throws SQLException, ItemException;

  /**
    replace an item in the Cache 

    @param c the item to replace (the current item with the same id
      as c will be replaced with c)
    @exception NoSuchItemException if there is no current item with
      the same id as c
    @exception SQLException if there is a problem with the database
    @exception ItemAccessException if the user cannot replace the item
   */
  public void replace(Cacheable c) 
      throws SQLException, NoSuchItemException, ItemAccessException;

  /**
    get an item from the Cache 

    @param id ID of the item
    @return The Cacheable item with the same id
    @exception NoSuchItemException if no item exists with that id
   */
  public Cacheable get(int id) throws NoSuchItemException;

  /**
    @return If the key refers to an object in the cache
    @param o key to check
   */
  public boolean containsKey(Object o);

  /**
    @return all the elements in the cache, in sorted order
   */
  public Iterator sortedElements();
}
