// java

package edu.washington.cac.calendar.data;

import java.sql.SQLException;

/** Something that can be stored in a Cache
  @author Greg Barnes
  @version 2.2
 */
public abstract class Cacheable extends DBObject implements Comparable
{
  public static final int NO_ID = -1;
  public static final int DELETED_ID = 0;

  public abstract int getId();
  public abstract void add(User user) throws SQLException;

  /** Delete this object from the database */
  public abstract void delete(User user) throws SQLException;

  public abstract void replace(User user) throws SQLException;

  /** Is this object less than another
    @param c Object to compare to
    @return Is this object less than the other
   */
  public abstract boolean lessThan(Cacheable c);

  /**
    Compares this object with the specified object for order.

    @param o Object to compare to.  If o is Cacheable but not of
      the same type as this object, the result will be consistent
      but not necessarily useful.
    @return a negative integer, zero, or a positive integer as this object
      is less than, equal to, or greater than the specified object.
    @exception ClassCastException If o is not a Cacheable object
   */
  public int compareTo(Object o) throws ClassCastException
  {
    Cacheable c = (Cacheable) o;

    if (this.lessThan(c)) {
      return -1;
    } else if (c.lessThan(this)) {
      return 1;
    } else {
      return 0;
    }
  }
}
