/*++DurationProp.java+++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* Log:
 * see eof
 */


package  edu.washington.cac.calendar.icalendar;


import  java.text.DecimalFormat;
import  java.text.ParsePosition;
import  java.text.ParseException;


/**
 * Base class for iCalendar DURATION-valued properties.
 *
 *<PRE>	<!-- for alignment purposes -->&nbsp;
 * Spec:
 *   section:	4.3.6 [2445]
 *   allowed:	---
 *</PRE>
 *
 * @author  slh
 * @version  0.00 2002/12/10 slh
 */
abstract public
class  DurationProp
extends  Property
{

/*----------------------------------------------------------------------------
 *						Constructors
 *--------------------------------------------------------------------------*/

  /*Note: attempting to use the default constructor is wrong*/


  /* this is the base constructor */
  /**
   * Create an DURATION property, unset.
   */
  protected
  DurationProp  (String	strName	)
  {
    super( strName );
  }


  /**
   * Create a DURATION property set to supplied value.
   */
  protected
  DurationProp  (String	strName		,
		 String	strValue	)
  {
    /*Note: letting calls handle any nulls*/

    this( strName );

    try {
      /*void*/set( strValue );
    } catch (Exception	e	) {
      throw new IllegalArgumentException( getName(  ) +
		": " + e.getLocalizedMessage(  ) );
    }
  }


/*----------------------------------------------------------------------------
 *						Object Public Methods
 *--------------------------------------------------------------------------*/

  public
  boolean
  isComplete  (		)
  {
//???
    return super.isComplete(  );
  }


  public
  int
  get  (		)
  {
    return m_Duration;
  }


  public
  void
  set  (int	Duration	)
  {
    m_Duration = Duration;
  }


  public
  void
  set  (String	strValue	)
  throws Exception
  {
    DecimalFormat	df		= new DecimalFormat(  );
/*???
    df = DecimalFormat.getNumberInstance(  );
*/
    ParsePosition	pp		= new ParsePosition( 0 );
    Number		number;
    int			Negative	= 1;
    int			Duration	= 0;
    String		strLast		= null;

    df.setParseIntegerOnly( true );
    pp.setIndex( 0 );

    if        (strValue.startsWith( "+" , pp.getIndex(  ) )) {
      pp.setIndex( pp.getIndex(  ) + 1 );
    } else if (strValue.startsWith( "-" , pp.getIndex(  ) )) {
      Negative = -1;
      pp.setIndex( pp.getIndex(  ) + 1 );
    }
    if (!strValue.startsWith( "P" , pp.getIndex(  ) )) {
      throw new ParseException( getName(  ) +
		": illegal duration format: " + strValue , pp.getIndex(  ) );
    }
    pp.setIndex( pp.getIndex(  ) + 1 );

   /* week or day portion */
    if (!strValue.startsWith( "T" , pp.getIndex(  ) )) {
      number = df.parse( strValue , pp );
      if (number == null) {
	throw new ParseException( getName(  ) +
		": illegal duration format: " + strValue , pp.getIndex(  ) );
      }
      Duration += number.intValue(  ) *
			getMultiplierWD( strValue , pp , null );
      if        (strValue.startsWith( "W" , pp.getIndex(  ) )) {
	if (strValue.length(  ) != pp.getIndex(  ) + 1) {
	  throw new ParseException( getName(  ) +
		": illegal duration format: " + strValue , pp.getIndex(  ) );
	}
      } else if (!strValue.startsWith( "D" , pp.getIndex(  ) )) {
	throw new ParseException( getName(  ) +
		": illegal duration format: " + strValue , pp.getIndex(  ) );
      }
      pp.setIndex( pp.getIndex(  ) + 1 );
    }

   /* time portion */
    if (strValue.startsWith( "T" , pp.getIndex(  ) )) {
      pp.setIndex( pp.getIndex(  ) + 1 );

      number = df.parse( strValue , pp );
      if (number == null) {
	throw new ParseException( getName(  ) +
		": illegal duration format: " + strValue , pp.getIndex(  ) );
      }
      Duration += number.intValue(  ) *
			getMultiplierHMS( strValue , pp , strLast );
      strLast = strValue.substring( pp.getIndex(  ) , pp.getIndex(  ) + 1 );
      pp.setIndex( pp.getIndex(  ) + 1 );

      if (strValue.length(  ) != pp.getIndex(  )) {
	number = df.parse( strValue , pp );
	if (number == null) {
	  throw new ParseException( getName(  ) +
		": illegal duration format: " + strValue , pp.getIndex(  ) );
	}
	Duration += number.intValue(  ) *
			getMultiplierHMS( strValue , pp , strLast );
	strLast = strValue.substring( pp.getIndex(  ) , pp.getIndex(  ) + 1 );
	pp.setIndex( pp.getIndex(  ) + 1 );
      }

      if (strValue.length(  ) != pp.getIndex(  )) {
	number = df.parse( strValue , pp );
	if (number == null) {
	  throw new ParseException( getName(  ) +
		": illegal duration format: " + strValue , pp.getIndex(  ) );
	}
	Duration += number.intValue(  ) *
			getMultiplierHMS( strValue , pp , strLast );
	strLast = strValue.substring( pp.getIndex(  ) , pp.getIndex(  ) + 1 );
	pp.setIndex( pp.getIndex(  ) + 1 );
      }
    }

    if (strValue.length(  ) != pp.getIndex(  )) {
      throw new ParseException( getName(  ) +
		": illegal duration format: " + strValue , pp.getIndex(  ) );
    }

    m_Duration = Negative * Duration;
  }

  
  public
  String
  getString  (		)
  {
    int		Duration	= m_Duration;
    String	strDuration	= "";
    int		Tmp;

    if (Duration < 0) {
      strDuration += "-";
      Duration = -Duration;
    }
    strDuration += "P";

    if (Duration % mc_SecsPerWeek == 0) {
      strDuration += Duration / mc_SecsPerWeek + "W";
      Duration = 0;
    }	/* and done */

    if (Duration >= mc_SecsPerDay) {
      Tmp = Duration / mc_SecsPerDay;
      strDuration += Tmp + "D";
      Duration -= Tmp * mc_SecsPerDay;
    }
    if (Duration != 0) {
      strDuration += "T";
    }
    if (Duration >= mc_SecsPerHour) {
      Tmp = Duration / mc_SecsPerHour;
      strDuration += Tmp + "H";
      Duration -= Tmp * mc_SecsPerHour;
    }
    if (Duration >= mc_SecsPerMinute) {
      Tmp = Duration / mc_SecsPerMinute;
      strDuration += Tmp + "M";
      Duration -= Tmp * mc_SecsPerMinute;
    }
    if (Duration > 0) {
      strDuration += Duration + "S";
      Duration = 0;
    }

    return strDuration;
  }


/*----------------------------------------------------------------------------
 *						Object Private Methods
 *--------------------------------------------------------------------------*/

  private
  int
  getMultiplierWD  (String		strValue	,
		    ParsePosition	pp		,
		    String		strLast		)	/* not used */
  throws ParseException
  {
    if        (strValue.startsWith( "W" , pp.getIndex(  ) )) {
      return mc_SecsPerWeek;
    } else if (strValue.startsWith( "D" , pp.getIndex(  ) )) {
      return mc_SecsPerDay;
    } else {
      throw new ParseException( getName(  ) +
		": illegal duration format: " + strValue , pp.getIndex(  ) );
    }
  }


  private
  int
  getMultiplierHMS  (String		strValue	,
		     ParsePosition	pp		,
		     String		strLast		)
  throws ParseException
  {
    if        (strValue.startsWith( "H" , pp.getIndex(  ) ) &&
	       strLast == null) {
      return mc_SecsPerHour;
    } else if (strValue.startsWith( "M" , pp.getIndex(  ) ) &&
	       (strLast == null || strLast.startsWith( "H" ))) {
      return mc_SecsPerMinute;
    } else if (strValue.startsWith( "S" , pp.getIndex(  ) )) {
      return 1;
    } else {
      throw new ParseException( getName(  ) +
		": illegal duration format: " + strValue , pp.getIndex(  ) );
    }
  }


/*----------------------------------------------------------------------------
 *						Class Protected Constants
 *--------------------------------------------------------------------------*/

  static protected final  int	mc_SecsPerMinute	= 60;
  static protected final  int	mc_SecsPerHour		=
							60 * mc_SecsPerMinute;
  static protected final  int	mc_SecsPerDay		= 24 * mc_SecsPerHour;
  static protected final  int	mc_SecsPerWeek		= 7 * mc_SecsPerDay;


/*----------------------------------------------------------------------------
 *						Object Protected Variables
 *--------------------------------------------------------------------------*/

//???use Integer
  protected  int		m_Duration	= 0;	/* in seconds */


/*----------------------------------------------------------------------------
 *						Main
 *--------------------------------------------------------------------------*/

  static public
  void
  main  (String	argv[]	)
  {
    Duration	duration	= new Duration(  );
    int		Value;
    String	strValue;

    try {
      Value = 3872;
      System.out.println( "int: " + Value );
      duration.set( Value );
      System.out.println( duration );
      strValue = "-P11W";
      System.out.println( "string: " + strValue );
      duration.set( strValue );
      System.out.println( duration );
      strValue = "-P13DT5H4M37S";
      System.out.println( "string: " + strValue );
      duration.set( strValue );
      System.out.println( duration );
      strValue = "PT37S";
      System.out.println( "string: " + strValue );
      duration.set( strValue );
      System.out.println( duration );
    } catch (ParseException	e	) {
      System.out.println( "error index: " + e.getErrorOffset(  ) );
      e.printStackTrace(  );
    } catch (Exception	e	) {
      e.printStackTrace(  );
    }
  }

}


/* Log:
 *  0.10  2002/12/10  slh
 *        redo constructors
 *        drop toString() (let super do with getString())
 *        isComplete()
 *  0.00  2001/08/20 - 2001/08/21  slh
 *        create from TimeProp.java
 */
/*--DurationProp.java-------------------------------------------------------*/
