/*++TextListParam.java++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* Log:
 * see eof
 */


package  edu.washington.cac.calendar.icalendar;


import  java.util.Iterator;
import  java.util.StringTokenizer;
import  java.util.Vector;


/**
 * Base class for TEXT-valued parameters
 * that allow a delimiter-separated list of texts.
 *
 * Subclassing:
 * override AddDirect() and AddTextDirect()
 * to make a class with different constraints on individual members of list.
 *
 * @author  slh
 * @version  0.10 2002/11/06 slh
 */
abstract public
class  TextListParam
extends  TextParam
{

/*----------------------------------------------------------------------------
 *						Constructors
 *--------------------------------------------------------------------------*/

  /*Note: attempting to use the default constructor is wrong*/


  /**
   * Create a TextList parameter unset.
   */
  /* Note: this is the base constructor */
  protected
  TextListParam  (String	strName	)
  {
    super( strName );

    m_vector = new Vector(  );
  }


  /**
   * Create a TextList parameter set to supplied value.
   */
  /* Convenience */
  protected
  TextListParam  (String	strName		,
		  String	strValues	)
  {
    /*Note: letting calls handle any nulls*/

    this( strName );

    try {
      /*void*/set( strValues );
    } catch (Exception	e	) {
      throw new IllegalArgumentException( e.getLocalizedMessage(  ) );
    }
  }


  /**
   * Create a TextList parameter unset and
   * set minimum and maximum number of elements.
   */
  protected
  TextListParam  (String	strName	,
		  int		Min	,
		  int		Max	)
  {
    /*Note: letting calls handle any nulls*/

    this( strName );

    /*void*/setMinMax( Min , Max );
  }


  /**
   * Create a TextList parameter unset and
   * set the delimiter and the minimum and maximum number of elements.
   */
  /* Convenience */
  protected
  TextListParam  (String	strName		,
		  String	strDelim	,
		  int		Min		,
		  int		Max		)
  {
    /*Note: letting calls handle any nulls*/

    this( strName );

    /*void*/setDelim( strDelim );
    /*void*/setMinMax( Min , Max );
  }


/*----------------------------------------------------------------------------
 *						Object Public Methods
 *--------------------------------------------------------------------------*/

  public
  boolean
  isComplete  (		)
  {
    /*Note: second clause should always be true*/
    return m_Min <= m_vector.size(  ) &&
		(m_Max == UNBOUNDED || m_vector.size(  ) <= m_Max);
  }


  /**
   * @return  count of values in list
   */
  public
  int
  getCount  (		)
  {
    return m_vector.size(  );
  }


  /***
   * Get Iterator of correct object class for class (String here).
   * @return  iterator of correct object class for class (String here)
   */
  public
  Iterator
  getList  (		)
  {
    return m_vector.iterator(  );
  }


  /**
   * @return  first value as object (which is a String here)
   */
  /* Convenience */
  public
  Object
  getFirst  (		)
  throws NoValueException
  {
    Iterator		enum;

    enum = m_vector.iterator(  );

    if (enum.hasNext(  )) {
      return enum.next(  );
    } else {
      throw new NoValueException( getName(  ) + ": no values" );
    }
  }


  /**
   * @return  first value as a String
   */
  /* Convenience */
  public
  String
  getFirstText  (		)
  throws NoValueException
  {
    /* probably work for all classes */
    return (String)getFirst(  );
  }


  /**
   * @param  strValues	comma-separated list of values
   */
  public
  void
  set  (String	strValues	)
  throws Exception
  {
    /*Note: letting calls take care of null cases*/

    m_vector.removeAllElements(  );

    addTexts( strValues );
  }


  /**
   * @param
   *	enum	iterator of correct object class for class (String here)
   */
  /* convenience */
  public
  void
  set  (Iterator	enum	)
  throws Exception
  {
    /*Note: letting calls take care of null cases*/

    m_vector.removeAllElements(  );

    for ( ; enum.hasNext(  ) ; ) {
      add( enum.next(  ) );
    }
  }


  /**
   * @param  object	object of correct class for class (String here)
   */
  public
  void
  add  (Object	object	)
  throws Exception	/* TooManyValuesException, NullPointerException */
  {
    if (object == null) {
      throw new NullPointerException(
		getName(  ) +
		": object (Object) is null" );
    }

    if (m_Max == UNBOUNDED || m_vector.size(  ) < m_Max) {
      /*Note: addDirect() appropriate for current subclass is called*/
      addDirect( object );
    } else {
      throw new TooManyValuesException(
		getName(  ) +
		": attempt to add more than maximum allowed values" );
    }
  }


  public
  void
  addText  (String	strText	)
  throws Exception	/* TooManyValuesException, NullPointerException */
  {
    if (strText == null) {
      throw new NullPointerException(
		getName(  ) +
		": strText (String) is null" );
    }

    if (m_Max == UNBOUNDED || m_vector.size(  ) < m_Max) {
      /*Note: addTextDirect() appropriate for current subclass is called*/
      addTextDirect( strText );
    } else {
      throw new TooManyValuesException(
		getName(  ) +
		": attempt to add more than maximum allowed values" );
    }
  }


  /**
   * @param  strValues	comma-separated list of values
   */
  /* Convenience+ */
  public
  void
  addTexts  (String	strTexts	)
  throws Exception	/* TooManyValuesException */
  {
/*???may need to use StreamTokenizer and catch quoted strings*/
    /*Note: lets constructor catch nulls*/
    StringTokenizer	st;

    if (strTexts == null) {
      throw new NullPointerException(
		getName(  ) +
		": strTexts (String) is null" );
    }

    st = new StringTokenizer( strTexts , m_strDelim , false );
    for ( ; st.hasMoreTokens(  ) ; ) {
      addText( st.nextToken(  ) );
    }
  }


  public
  String
  getString  (		)
  {
/*IMPLEMENT: more effeciently*/
    int		idx;
    String	str		= "";

    for (idx = 0 ; idx < m_vector.size(  ) ; idx++) {
      if (idx != 0) {
	str += m_strDelim;
      }
      str += m_vector.elementAt( idx );
    }

    return str;
  }


/*----------------------------------------------------------------------------
 *						Class Public Constants
 *--------------------------------------------------------------------------*/

  final static public  int	UNBOUNDED	= -1;


/*----------------------------------------------------------------------------
 *						Object Protected Methods
 *--------------------------------------------------------------------------*/

  /**
   * Add text after null and value count checks.
   * Should be overridden by any subclasses using other value types.
   */
  protected
  void
  addDirect  (Object	object	)
  throws Exception
  {
    /* probably work for all classes, though not sensible for all that work */
    addTextDirect( (String)object );
  }


  /**
   * Add text after null and value count checks.
   * Should be overridden by any subclasses using other value types.
   */
  protected
  void
  addTextDirect  (String	strText	)
  throws Exception
  {
/*???check that text is safe (no bare `,', etc */
    m_vector.addElement( strText );
  }


  protected
  void
  setDelim  (String	strDelim	)
  {
    m_strDelim = strDelim;
  }


  protected
  void
  setMinMax  (int	Min	,
	      int	Max	)
  {
    /* if caller wants to set them to something goofy, let them */
    m_Min = Min;
    m_Max = Max;
  }


/*----------------------------------------------------------------------------
 *						Object Protected Variables
 *--------------------------------------------------------------------------*/

  /*Assert: m_vector never null after construction*/
  protected  Vector	m_vector	= null;
  /*Assert: these never change after construction*/
  protected  int	m_Min		= 0;
  protected  int	m_Max		= UNBOUNDED;
  protected  String	m_strDelim	= ",";

}


/* Log:
 *  0.10  2002/11/06  slh
 *        (resync'ing w/TextListProp:)
 *        make delim settable (at construction)
 *        make min/max only settable at construction
 *        redo constructors
 *        comments
 *        getCount()
 *  0.01  2001/05/21 - 2001/05/22  slh
 *        getFirst(), getFirstText()
 *        getTextList() -> getList()
 *        min/max
 *        object vars were decl static
 *  0.00  2001/05/16 - 2001/05/17  slh
 *        create
 */
/*--TextListParam.java------------------------------------------------------*/
