/*++VEvent.java+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* Log:
 * see eof
 */


package  edu.washington.cac.calendar.icalendar;


import  java.util.Vector;


/**
 * iCalendar VEVENT component.
 *
 *<PRE>	<!-- for alignment purposes -->&nbsp;
 * Spec:
 *   section:	4.6.1 [2445]
 *   occurs:	VCALENDAR (0+)
 *   comps:	VALARM (0+)
 *   props:	CLASS(0-1), CREATED(0-1), DESCRIPTION(0-1), DTSTART(0-1),
 *		GEO(0-1), LAST-MODIFIED(0-1), LOCATION(0-1), ORGANIZER(0-1),
 *		PRIORITY(0-1), DTSTAMP(0-1), SEQUENCE(0-1), STATUS(0-1),
 *		SUMMARY(0-1), TRANSP(0-1), UID(0-1), URL(0-1),
 *		RECURRENCE-ID(0-1),
 *		*DTEND(0-1), DURATION(0-1)*,
 *		ATTACH(0+), ATTENDEE(0+), CATEGORIES(0+), COMMENT(0+),
 *		CONTACT(0+), EXDATE(0+), EXRULE(0+), REQUEST-STATUS(0+),
 *		RELATED-TO(0+), RESOURCES(0+), RDATE(0+), RRULE(0+),
 *		x-prop(0+)
 *
 * *only one of DTEND/DURATION may appear
 *</PRE>
 *
 * @author  slh
 * @version  0.10 2003/04/21 slh
 */
public
class  VEvent
extends  Component
{

/*----------------------------------------------------------------------------
 *						Constructors
 *--------------------------------------------------------------------------*/

  public
  VEvent  (		)
  {
    super( getCalendarName(  ) );
  }


/*----------------------------------------------------------------------------
 *						Class Public Methods
 *--------------------------------------------------------------------------*/

  static public
  String
  getCalendarName  (		)
  {
    return Names.strComp_VEvent;
  }


/*----------------------------------------------------------------------------
 *						Object Public Methods
 *--------------------------------------------------------------------------*/

  public
  void
  addProperty  (Property	property	)
  throws ICalendarException
  {
    java.lang.Class	jlcDTEnd, jlcDuration;

    if (property == null) {
      throw new NullPointerException( getName(  ) + ": addProperty()" +
		": property (Property) is null" );
    }

    jlcDTEnd = Config.getPropClass( Names.strProp_DTEnd );
    jlcDuration = Config.getPropClass( Names.strProp_Duration );

    /* if property is a DTEND or DURATION and either is already present... */
    if ((jlcDTEnd.isInstance( property ) ||
	 jlcDuration.isInstance( property )) &&
	(0 != m_properties.getCount( jlcDTEnd ) ||
	 0 != m_properties.getCount( jlcDuration ))) {
      throw new InvalidPropertyException( getName(  ) + ": addProperty()" +
		": only one instance of either DTEND or DURATION allowed" );
    }

    /*void*/super.addProperty( property );
  }


/*----------------------------------------------------------------------------
 *						Object Protected Methods
 *--------------------------------------------------------------------------*/

  protected
  void
  configComponent  (		)
  {
    m_vCompZeroPlus = new Vector(  );
    /*void*/m_vCompZeroPlus.add( VAlarm.class );

    m_vPropZeroOne = new Vector(  );
    /*void*/m_vPropZeroOne.add( Class.class );
    /*void*/m_vPropZeroOne.add( Created.class );
    /*void*/m_vPropZeroOne.add( Description.class );
    /*void*/m_vPropZeroOne.add( DTStart.class );
    /*void*/m_vPropZeroOne.add( Geo.class );
    /*void*/m_vPropZeroOne.add( LastModified.class );
    /*void*/m_vPropZeroOne.add( Location.class );
    /*void*/m_vPropZeroOne.add( Organizer.class );
    /*void*/m_vPropZeroOne.add( Priority.class );
    /*void*/m_vPropZeroOne.add( DTStamp.class );
    /*void*/m_vPropZeroOne.add( Sequence.class );
    /*void*/m_vPropZeroOne.add( Status.class );
    /*void*/m_vPropZeroOne.add( Summary.class );
    /*void*/m_vPropZeroOne.add( Transp.class );
    /*void*/m_vPropZeroOne.add( UID.class );
    /*void*/m_vPropZeroOne.add( URL.class );
    /*void*/m_vPropZeroOne.add( RecurrenceId.class );
    /*void*/m_vPropZeroOne.add( DTEnd.class );
    /*void*/m_vPropZeroOne.add( Duration.class );

    m_vPropZeroPlus = new Vector(  );
    /*void*/m_vPropZeroPlus.add( Attach.class );
    /*void*/m_vPropZeroPlus.add( Attendee.class );
    /*void*/m_vPropZeroPlus.add( Categories.class );
    /*void*/m_vPropZeroPlus.add( Comment.class );
    /*void*/m_vPropZeroPlus.add( Contact.class );
    /*void*/m_vPropZeroPlus.add( ExDate.class );
    /*void*/m_vPropZeroPlus.add( ExRule.class );
    /*void*/m_vPropZeroPlus.add( RequestStatus.class );
    /*void*/m_vPropZeroPlus.add( RelatedTo.class );
    /*void*/m_vPropZeroPlus.add( Resources.class );
    /*void*/m_vPropZeroPlus.add( RDate.class );
    /*void*/m_vPropZeroPlus.add( RRule.class );
    /*void*/m_vPropZeroPlus.add( XProp.class );
  }

}


/* Log:
 *  0.10  2003/02/12, 2003/02/18, 2003/04/21  slh
 *        more or less complete redo to conform with current patterns
 *  0.00  2001/04/24, 2001/05/01, 2001/05/09  slh
 *        create
 */
/*--VEvent.java-------------------------------------------------------------*/
