// Java

/* log:
 * 2000/05/30  slh  log/logcommon() was not done correctly for Type
 * ...
 * 2000/03/13  slh  Log()s w/o uid params; add log/loguid(); ch output
 */

package edu.washington.cac.myuw.servlet;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.io.Serializable;

import edu.washington.cac.myuw.util.CurrTime;

  /** A log for the servlet to print to, parameterized for each user
    @author FangLin, Greg Barnes
    @version 2.0
  */
public class Log implements Serializable {
/*
private void writeObject(java.io.ObjectOutputStream out)
     throws IOException{}
 private void readObject(java.io.ObjectInputStream in)
     throws IOException, ClassNotFoundException {}
*/
    private static String noUidString = "[global]";

  private String uid = null;

  /** The underlying PrintStream to print to */
  private PrintStream log = null;
 
    public static final int     LOGTYPE_INFO            = 0;
    public static final int     LOGTYPE_NOTICE          = 1;
    public static final int     LOGTYPE_WARNING         = 2;
    public static final int     LOGTYPE_ERROR           = 4;

  /** Create a new log for a user
    @param uid Printable id of the user
    @param log <code>java.io.PrintStream</code> to use as output
   */
  public Log (String uid, PrintStream log)
  {
    this.uid = uid;

    if (log == null) {
      this.log = Logfile.log;
    } else {
      this.log = log;
    }
  }

  /** Create a new log for a user, using the default output file
    @param uid Printable id of the user
   */
  public Log(String uid)
  {
    this(uid, null);
  }

  /** Create a new log with a default user, using the default output file
   */
  public Log ()
  {
    this(null, null);
  }

  /** Create a new log with a default user
    @param log <code>java.io.PrintStream</code> to use as output
   */
  public Log(PrintStream log)
  {
    this(null, log);
  }

  /**
    Get the underlying PrintStream.  Useful for printing stack traces.
    @return The underlying PrintStream
   */
  public PrintStream getPrintStream() {
    return log;
  }

  /**
    set the string that is printed when no uid is associated with the log
    @param noUidString The string to print when no uid is associated with
        the log
   */
  public static void setNoUidString(String s) {
    noUidString = s;
  }

  /**
    @return the user id for this log
   */
  public String getUid()
  {
    return this.uid;
  }

  /**
    Set the user id for this log
    @param uid The user id
   */
  public void setUid(String uid)
  {
    this.uid = uid;
  }

  public void println(boolean x) {
    StringBuffer s = outcommon();
    s.append(x);
    this.log.println(s.toString());
  }

  public void println(int x) {
    StringBuffer s = outcommon();
    s.append(x);
    this.log.println(s.toString());
  }


  public void println(long x) {
    StringBuffer s = outcommon();
    s.append(x);
    this.log.println(s.toString());
  }


    public void println(Object x) {
        StringBuffer s = outcommon();
        s.append(x);
        this.log.println(s.toString());
    }

    private StringBuffer blanks(int length) {
        StringBuffer s = new StringBuffer();
        int i;

        for (i = 0; i < length; i++) {
            s.append(" ");
        }

        return s;
    }

    private String forceLength(String s, 
                                int length) 
    {
        if (s.length() > length) {
            return s.substring(0, length);
        } else if (s.length() < length) {
            return s + blanks(length - s.length());
        } else {
            return s;
        }
    }
        
        
    private StringBuffer outcommon()
    {
        String strUID = (this.uid == null) ? this.noUidString : this.uid;
        StringBuffer s = new StringBuffer("  ");
        s.append(forceLength(strUID, 8));
        s.append(" ");
        s.append(CurrTime.getCompTime());
        s.append(" ");
        return s;
    }


    public  void  log  (String  strModule       ,
                        String  strMethod       ,
                        int     Type            ,
                        String  strText         )
    {
        switch (Type) {
        case LOGTYPE_ERROR:
        case LOGTYPE_WARNING:
        case LOGTYPE_NOTICE:
            StringBuffer s = outcommon();
            s.append( logcommon( strModule , strMethod , Type , strText ));
            this.log.println( s.toString(  ) );
            break;
        case LOGTYPE_INFO:
        default:
        }
    }

    public StringBuffer  logcommon  (String     strModule       ,
                    String      strMethod       ,
                    int         Type            ,
                    String      strText         )
    {
        StringBuffer s = new StringBuffer();

        switch (Type) {
        case LOGTYPE_ERROR:
        case LOGTYPE_WARNING:
        case LOGTYPE_NOTICE:
            s.append( strModule );
            s.append( ":" );
            s.append( strMethod );
            s.append( "():" );
            break;
        case LOGTYPE_INFO:
        default:
            break;
        }

        switch (Type) {
        case LOGTYPE_ERROR:
            s.append( "ERROR" );
            break;
        case LOGTYPE_WARNING:
            s.append( "WARNING" );
            break;
        case LOGTYPE_NOTICE:
            s.append( "NOTICE" );
            break;
        case LOGTYPE_INFO:
//          s.append( "INFO" );
            break;
        default:
//          s.append( Type );
            break;
        }

        switch (Type) {
        case LOGTYPE_ERROR:
        case LOGTYPE_WARNING:
        case LOGTYPE_NOTICE:
            s.append( ": " );
            s.append( strText );
            break;
        case LOGTYPE_INFO:
        default:
            break;
        }

        return s;
    }


  public static void main(String args[]) throws IOException 
  {
    PrintStream log = new PrintStream(new FileOutputStream("ss", true));
    new Log ("fanglin", log).println("Yes");
  }
}
