package pubevents.addevents;

import java.io.IOException;
import java.io.PrintWriter;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import edu.washington.cac.calendar.data.CaldataException;
import edu.washington.cac.calendar.data.Keyword;
import edu.washington.cac.calendar.data.NoSuchItemException;
import edu.washington.cac.calendar.data.User;
import edu.washington.cac.calendar.db.DBResources;
import edu.washington.cac.calendar.db.Fields;
import edu.washington.cac.calendar.db.Queries;
import edu.washington.cac.calendar.db.Tables;

import pubevents.PEDefaults;
import pubevents.PEParameter;
import pubevents.util.DataUtil;
import pubevents.util.PrintWebPage;

/** AddEvents: add a new event to database after editing is completed.
  
  @author Leman Chung, Greg Barnes
  @version 1.2
 */
public class AddEvents 
{
  /**
    Add an event
    @param req HTTP request object with form data for the event
    @param res HTTP response object
    @param uid Person trying to add the event
    @exception IOException If there's a problem writing the new web page
    @exception NoSuchItemException if the form references a keyword that
               doesn't exist
    @exception SQLException If there's a problem accessing the database
    @exception CaldataException If there's a problem with the form data
   */
  public AddEvents(HttpServletRequest req, HttpServletResponse res, String uid)
      throws IOException, SQLException, CaldataException, NoSuchItemException
  {
    int sponsorid, locationid, eventid;

    res.setContentType("text/html");
    PrintWriter out = res.getWriter();
    SqlStatements sqlstmts = new SqlStatements ();
    PEParameter pep = new PEParameter(req);

    /* if a sponsor with the same name, phone, e-mail, link, and creator exist
       in sponsors table then retrieve its sponsorid, otherwise add this 
       sponsor to the database and then retrieve its sponsorid.
     */
    try {    
      ResultSet rs = sqlstmts.selectSponsors(req);
  
      if (rs.next()) {
        sponsorid = rs.getInt(Fields.SSPONSOR_ID.getName());
      } else {
            // doesn't exist, so we add it to db table
        sponsorid = sqlstmts.addSponsors(req, uid);
      }
  
      /* Similarly, add the location if it doesn't exist */
      rs = sqlstmts.selectLocations(req);
  
      if (rs.next()) {
        locationid = rs.getInt(Fields.LLOCATION_ID.getName());
      } else {
        locationid = sqlstmts.addLocations(req, uid);
      }
  
      Keyword k = DataUtil.getKeyword(pep);

      // add the event itself
      eventid = sqlstmts.addEvents(req, uid, locationid, sponsorid);
  
      // finally, add the eventkeyword entry
      sqlstmts.addEventKeywords(eventid, k.getId());
    } finally {
      sqlstmts.close();
    }

    // print the event added message.
    new PrintWebPage(PEDefaults.EVENT_ADDED, out);
    out.close();
  }
}
