package pubevents.authtable;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import edu.washington.cac.calendar.db.Fields;
import edu.washington.cac.calendar.db.Tables;

import pubevents.AdminServlet;
import pubevents.PEDefaults;
import pubevents.addevents.SqlStatements;
import pubevents.util.PrintErrorMessage;
import pubevents.util.PrintWebPage;

/** List users from authorization table.
  
  @author Leman Chung
  @version 2.2
 */
public class ListUsers extends AdminServlet 
{
  public void performAction(HttpServletRequest req, HttpServletResponse res)
                 throws IOException {
    String firstname, lastname,
           phone, email, department;
    ResultSet rs = null;
    SqlStatements sqlstmts = null;
    String templateFile_Header = PEDefaults.HEADER;
    String templateFile_Footer = PEDefaults.FOOTER;
    String templateFile;

    PrintWriter out = res.getWriter();
    int counter = 0;
    String error_msg = "";
    String html_title = "Events Calendar: List all the users on Authorization table.";

          // print header
        templateFile = templateFile_Header;
    BufferedReader in = new BufferedReader(new InputStreamReader(
            this.getClass().getResourceAsStream(templateFile)));

        String aline = in.readLine(); // IOException
        while (aline != null) {
          if (aline.startsWith("%%%html_title")) {
              out.println(html_title);
          } else if (aline.startsWith("%%%error_msg")) {
              out.println(error_msg);
          } else if (aline.startsWith("%%%")) {
              // skip this line
          } else {
              out.println(aline);
          }
          aline = in.readLine();
        }


        sqlstmts = new SqlStatements();
        try {
             rs = sqlstmts.executeQuery("select "
                                        + Fields.UUSERNAME.getSelectName()
                                        + ", "
                                        + Fields.AFIRSTNAME.getSelectName()
                                        + ", "
                                        + Fields.ALASTNAME.getSelectName()
                                        + ", "
                                        + Fields.APHONE.getSelectName() + ", "
                                        + Fields.AEMAIL.getSelectName() + ", "
                                        + Fields.ADEPARTMENT.getSelectName() 
                                        + ", "
                                        + Fields.AUSERTYPE.getSelectName()
                                        + " from "
                                        + Tables.AUTH.getName()
                                        + ", "
                                        + Tables.USERS.getName()
                                        + " where "
                                        + Fields.AUSERID.getStmtName()
                                        + " = "
                                        + Fields.UUSER_ID.getStmtName()
                                        + " order by "
                                        + Fields.UUSERNAME.getStmtName());
        } catch (SQLException e) {
            e.printStackTrace(System.err);
            System.err.println("SQL Exception: " + e.getMessage());
            error_msg =  e.getMessage();
        } catch (Exception e) {
            e.printStackTrace(System.err);
            System.err.println("Exception: " + e.getMessage());
            error_msg =  e.getMessage();
        }

        try {
          if (error_msg == "") {
             out.println("<table>");
             out.println("<tr><th>NetID</th><th>Name</th><th>Phone No.</th><th>E-mail</th><th>Department</th><th>User Type</th></tr>");
             while (rs.next()) {
                firstname = "";
                lastname  = "";
                phone = "";
                email = "";
                department = "";
                if (rs.getString(Fields.AFIRSTNAME.getName()) != null) {
                    firstname = rs.getString(Fields.AFIRSTNAME.getName());
                }
                if (rs.getString(Fields.ALASTNAME.getName()) != null) {
                    lastname = rs.getString(Fields.ALASTNAME.getName());
                }
                if (rs.getString(Fields.APHONE.getName()) != null) {
                    phone = rs.getString(Fields.APHONE.getName());
                }
                if (rs.getString(Fields.AEMAIL.getName()) != null) {
                    email = rs.getString(Fields.AEMAIL.getName());
                }
                if (rs.getString(Fields.ADEPARTMENT.getName()) != null) {
                    department = rs.getString(Fields.ADEPARTMENT.getName());
                }

                out.println("<tr><td>" + rs.getString(Fields.UUSERNAME.getName()) + "&nbsp;</td>");
                out.println("<td>" + firstname + "&nbsp;" + lastname + "&nbsp;</td>");
                out.println("<td>" + phone + "&nbsp;</td>");
                out.println("<td>" + email + "&nbsp;</td>");
                out.println("<td>" + department + "&nbsp;</td>");
                out.println("<td>&nbsp;" + rs.getString(Fields.AUSERTYPE.getName()) + "</td></tr>");
             }
             out.println("</table>");
          }
        } catch (SQLException e) {
            e.printStackTrace(System.err);
            System.err.println("SQL Exception: " + e.getMessage());
            error_msg = e.getMessage();
        } catch (Exception e) {
            e.printStackTrace(System.err);
            System.err.println("Exception: " + e.getMessage());
            error_msg =  e.getMessage();
        } finally {
          sqlstmts.close();
        }

    if (error_msg != "") {
        new PrintErrorMessage().printError(out, "ERROR: " + error_msg);
    }

    templateFile = templateFile_Footer;
    new PrintWebPage(templateFile, out);
    out.close();
  }
}
