package pubevents.delevents;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.db.DBResources;
import edu.washington.cac.calendar.db.Fields;
import edu.washington.cac.calendar.db.PubEventsQueries;
import edu.washington.cac.calendar.db.Tables;

import pubevents.AdminServlet;
import pubevents.PEDefaults;
import pubevents.PEParameter;
import pubevents.addevents.SqlStatements;
import pubevents.util.PrintErrorMessage;
import pubevents.util.PrintWebPage;

/** AdminDelViaDates: Edit date range and then extract events for
                    administrator's later deletion.
  
  @author Leman Chung
  @version 2.0
 */
public class AdminDelViaDates extends AdminServlet 
{
  public void performAction(HttpServletRequest req, HttpServletResponse res)
      throws IOException
  {
    String smon, sday, syear,
           emon, eday, eyear;
    String range_begin = null, range_end = null;
    String qry_statement;
    ResultSet rs = null;
    SqlStatements sqlstmts = new SqlStatements();

    String templateDelEvents = PEDefaults.ADMIN_DEL_EVENTS;
    String templateUpdtViaDates = PEDefaults.UPDT_VIA_DATES;
    String templateFooter = PEDefaults.FOOTER;
    String templateFile;

    PEParameter pep = new PEParameter(req);
    PrintWriter out = res.getWriter();
    int counter = 0;
    String error_msg = "";

    String html_title = "Events Calendar: Delete Other People's Public Events via Date Range.";

    if (req.getParameter(CalParameter.START_MONTH).trim().equals("None")) {
      error_msg = "Start month is not selected!";
    } else if (req.getParameter(CalParameter.END_MONTH).trim().equals("None")) {
      error_msg = "End month is not selected!";
    }

    range_begin = pep.getStartYear() + "-" + 
                  sqlstmts.monthNumber(pep.getStartMonth()) + "-" + 
                  pep.getStartDay();
    range_end = pep.getEndYear() + "-" + 
                sqlstmts.monthNumber(pep.getEndMonth()) + "-" + 
                pep.getEndDay();

    if (error_msg == "") {
      qry_statement = "select " 
                      + Fields.EEVENT_ID.getSelectName() + ", " 
                      + Fields.ESTARTDATE.getSelectName() + ", " 
                      + Fields.EENDDATE.getSelectName() + ", "
                      + Fields.ESHORTDESC.getSelectName() + ", "
                      + Fields.ELONGDESC.getSelectName() + ", "
                      + Fields.ELONGDESC1.getSelectName() + " from "
                      + Tables.EVENTS.getName() + ", "
                      + Tables.EVENTKEYWORDS.getName() + ", "
                      + Tables.KEYWORDS.getName() + " "
                      + new PubEventsQueries().baseStandardWhere() + " and "
                      + Fields.EEVENT_ID.getStmtName() + "=" 
                      + Fields.EKEVENT_ID.getStmtName() + " and "
                      + Fields.EKKEYWORD_ID.getStmtName() + "="
                      + Fields.KKEYWORD_ID.getStmtName() + " and "
                      + Fields.ESTARTDATE.getStmtName() + " >= \'" 
                      + range_begin + "\' and " 
                      + Fields.ESTARTDATE.getStmtName() + " <= \'" 
                      + range_end 
                      + "\' order by " 
                      + Fields.ESTARTDATE.getStmtName() + " desc";

      try {
        rs = sqlstmts.executeQuery(qry_statement);
      } catch (Exception e) {
        e.printStackTrace(System.err);
        System.err.println("Exception: " + e.getMessage());
        error_msg = "ERROR: " + e.getMessage();
      }

      try {
        if (error_msg == "") {
          if (!rs.next()) {
            error_msg = "No records exist in this date range.";
          } else {
            new PrintWebPage(templateDelEvents, out); // print out header

            do {
              out.println("<tr><td><input type=\"checkbox\" name=\"" + 
                          PEParameter.EVENT_IDS + "\" value=\"" + 
                          rs.getString(Fields.EEVENT_ID.getName()) + 
                          "\"></td>");
              out.println("<td>" + rs.getDate(Fields.ESTARTDATE.getName()) + 
                          "</td>");
              out.println("<td>" + rs.getDate(Fields.EENDDATE.getName()) + 
                          "</td>");
              out.println("<td>" + rs.getString(Fields.ESHORTDESC.getName()) + 
                          "</td>");
              out.print("<td>" + Fields.getEventLongdesc(rs));
              out.println("</td></tr>");
            } while (rs.next());

            out.println("<tr><td colspan=\"5\"></td></tr>");
            out.println("<tr><td></td><td></td>");
            out.println("<td><input type=\"submit\" value=\"Submit\"></td>");
            out.println("<td><input type=\"reset\" value=\"Clear\"></td>");
            out.println("<td></td></tr>");
            out.println("</table>\n</form>"); 
            new PrintWebPage(templateFooter, out); // print out footer
          }
        }
      } catch (Exception e) {
          e.printStackTrace(System.err);
          System.err.println("Exception: " + e.getMessage());
          error_msg = "ERROR: " + e.getMessage();
      } finally {
        sqlstmts.close();
      }
    }

    // print out error message, such as start month is not selected.
    if (error_msg != "") {
      smon = pep.getStartMonth().substring(0, 3);
      sday = pep.getStartDay();
      syear = pep.getStartYear();
      emon = pep.getEndMonth().substring(0, 3);
      eday = pep.getEndDay();
      eyear = pep.getEndYear();

      templateFile = templateUpdtViaDates;
      BufferedReader in = new BufferedReader(new InputStreamReader(
              this.getClass().getResourceAsStream(templateFile)));

      String aline = in.readLine(); // IOException

      while (aline != null) {
        if (aline.startsWith("%%%html_title")) {
          out.println(html_title);
        } else if (aline.startsWith("%%%error_msg")) {
          new PrintErrorMessage().printError(out, error_msg);
        } else if (aline.startsWith("%%%action")) {
          out.println("\"pubevents.delevents.AdminDelViaDates\"");
        } else if (aline.startsWith("%%%" + CalParameter.FUNCTION)) {
          out.println("\"delete\"");
        } else if (aline.startsWith("%%%sel_smon")) {
          if (aline.indexOf(smon) != -1) {
            out.println("selected");
          }
        } else if (aline.startsWith("%%%sel_sday")) {
          if (aline.indexOf(sday) != -1) {
            out.println("selected");
          }
        } else if (aline.startsWith("%%%sel_syear")) {
          if (aline.indexOf(syear) != -1) {
            out.println("selected");
          }
        } else if (aline.startsWith("%%%sel_emon")) {
          if (aline.indexOf(emon) != -1) {
            out.println("selected");
          }
        } else if (aline.startsWith("%%%sel_eday")) {
          if (aline.indexOf(eday) != -1) {
            out.println("selected");
          }
        } else if (aline.startsWith("%%%sel_eyear")) {
          if (aline.indexOf(eyear) != -1) {
            out.println("selected");
          }
        } else if (aline.startsWith("%%%")) {
          // skip this line
        } else {
          out.println(aline);
        }

        aline = in.readLine();
      }

      in.close();
    }

    out.close();
  }
}
