package pubevents.updtalerts;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.PrintWriter;

import java.sql.Date;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import edu.washington.cac.calendar.CalLogFactory;
import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.CaldataException;
import edu.washington.cac.calendar.data.DataChecker;

import pubevents.AdminServlet;
import pubevents.PEDefaults;
import pubevents.PEParameter;
import pubevents.addalerts.AddAlerts;
import pubevents.util.AlertSocketClient;
import pubevents.util.ConvertRoutines;
import pubevents.util.PrintErrorMessage;

/** add or edit an alert.
  
  @author Leman Chung
  @version 2.0
 */
public class EditAlerts extends AdminServlet 
{
  public void performAction(HttpServletRequest req, HttpServletResponse res)
                 throws IOException {
    boolean isUpdate;
    String html_title, uptype;
    String templateFile = PEDefaults.ADD_ALERTS;

    PEParameter pep = new PEParameter(req);
    PrintWriter out = res.getWriter();

    int counter = 0;
    String error_msg = "";

    if (pep.getFunction().equals(PEDefaults.UPDATE_FUNCTION)) {
        html_title = PEDefaults.ALERT_TITLE + "Update An Alert.";
        uptype = PEDefaults.UPDATE_FUNCTION;
        isUpdate = true;
    } else {
        html_title = PEDefaults.ALERT_TITLE + "Add A New Alert.";
        uptype = "add";
        isUpdate = false;
    }

    if (pep.getShortdesc() == null) {
        error_msg = error_msg + ", Title";
        counter++;
    }
    if (pep.getLongdesc() == null) {
        error_msg = error_msg + ", Description";
        counter++;
    }
    if (req.getParameter(CalParameter.START_MONTH).trim().equals("None")) {
        error_msg = error_msg + ", Start Date";
        counter++;
    }
    if (req.getParameter(CalParameter.END_MONTH).trim().equals("None")) {
        error_msg = error_msg + ", End Date";
        counter++;
    }

    String startdate = pep.getStartYear() + "-"
                       + pep.getStartMonth()
                       + "-" + pep.getStartDay();
    String enddate = pep.getEndYear() + "-"
                       + pep.getEndMonth()
                       + "-" + pep.getEndDay();
    try {
      new DataChecker().checkDateTimes(java.sql.Date.valueOf(startdate),
                                       pep.getStarttime(),
                                       java.sql.Date.valueOf(enddate),
                                       pep.getEndtime());
    } catch (CaldataException e) {
      new CalLogFactory().create().println(e);
      error_msg = error_msg +  e.getMessage() + "  ";
    }

    if (error_msg == "") {
           // move the process to add/update events over here
           // in order to catch error and print it if ever happened.
       if (isUpdate) {
            try {
                 new UpdtAlerts(req, res, getUid());
            } catch (Exception e) {
                 e.printStackTrace(System.err);
                 System.err.println(e.toString());
                 error_msg = "ERROR: " + e.toString(); 
            }
       } else {
            try {
                 new AddAlerts(req, res, getUid());
            } catch (Exception e) {
                 e.printStackTrace(System.err);
                 System.err.println(e.toString());
                 error_msg = "ERROR: " + e.toString(); 
            }
       } 
    }

    if (error_msg == "") {
      try {            // update dynamic contents and alerts.html page
        new AlertSocketClient().talk();
      } catch (Exception e) {
          e.printStackTrace(System.err);
          System.err.println(e.toString());
          error_msg = "ERROR: " + e.toString(); 
      }
    }

    if (error_msg != "") {
        if (counter > 1) {
            error_msg = "Fields " + error_msg.substring(2) + " are required!&nbsp;&nbsp;Please enter/select these fields and re-submit.";
        } else if (counter == 1) {
            error_msg = error_msg.substring(2) + " is required!&nbsp;&nbsp;Please enter/select this field and re-submit.";
        }

             // build a new method over here
        String field;

        // read and print out data entry form.
        BufferedReader in = new BufferedReader(new InputStreamReader(
                this.getClass().getResourceAsStream(templateFile)));

        String aline = in.readLine();
        while (aline != null) {
          if (aline.startsWith("%%%html_title")) {
              out.println(html_title);
          } else if (aline.startsWith("%%%error_msg")) {
              new PrintErrorMessage().printError(out, error_msg);
          } else if (aline.startsWith("%%%" + PEParameter.RED_ALERT)) {
               if (pep.getRedAlert()) {
                  out.println("checked");
               }
          } else if (aline.startsWith("%%%" + CalParameter.LONG_DESCRIPTION)) {
              out.println(pep.getLongdesc() == null ? "" : pep.getLongdesc());
          } else if (aline.startsWith("%%%reset")) {
              if (isUpdate) {
                  out.println("\"Reset (back to original)\"");
              } else {
                  out.println("\"Clear\"");
              }
          } else if (aline.startsWith("%%%" + CalParameter.START_MONTH)) {
            out.print("\"" + req.getParameter(CalParameter.START_MONTH) +
                      "\">");
            try {
              out.print(new MyCalendar(pep.getStartdate()).monthString());
            } catch (CaldataException e) {
              new CalLogFactory().create().println(e.getMessage());
              out.print("");
            }
          } else if (aline.startsWith("%%%" + CalParameter.END_MONTH)) {
            out.print("\"" + req.getParameter(CalParameter.END_MONTH) + "\">");
            try {
              out.print(new MyCalendar(pep.getEnddate()).monthString());
            } catch (CaldataException e) {
              new CalLogFactory().create().println(e.getMessage());
              out.print("");
            }
          } else if (aline.startsWith("%%%" + CalParameter.START_DAY)
                     || aline.startsWith("%%%" + CalParameter.START_YEAR)
                     || aline.startsWith("%%%" + CalParameter.END_DAY)
                     || aline.startsWith("%%%" + CalParameter.END_YEAR)) {
              field = aline.substring(3);
              out.print(req.getParameter(field));
          } else if (aline.startsWith("%%%")) {
              field = aline.substring(3);
              if (req.getParameter(field) != null) {
                  out.print("\"" + req.getParameter(field) + "\"");
              }
          } else {
              out.println(aline);
          }
          aline = in.readLine();
        }
        in.close();
    }
    out.close();
  }
}
