package pubevents.updtevents;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.db.Fields;
import edu.washington.cac.calendar.db.PubEventsQueries;
import pubevents.AdminServlet;
import pubevents.PEDefaults;
import pubevents.PEParameter;
import pubevents.addevents.SqlStatements;
import pubevents.util.PrintErrorMessage;
import pubevents.util.PrintWebPage;

/** Edit date range and then extract events for
                   administrator's updating to other people's public events.
  
  @author Leman Chung
  @version 2.0
 */
public class AdminUpViaDates extends AdminServlet 
{
  public void performAction(HttpServletRequest req, HttpServletResponse res)
                 throws IOException {
    String smon, sday, syear,
           emon, eday, eyear;
    String range_begin = null, range_end = null;
    ResultSet rs = null;
    SqlStatements sqlstmts = new SqlStatements();

    String templateUpdtEvents = PEDefaults.ADMIN_UPDT_EVENTS;
    String templateUpdtViaDates = PEDefaults.UPDT_VIA_DATES;
    String templateFooter = PEDefaults.FOOTER;
    String templateFile;

    PrintWriter out = res.getWriter();
    int counter = 0;
    String error_msg = "";
    PEParameter pep = new PEParameter(req);

    String html_title = "Events Calendar: Update Other People's Public Events via Date Range.";

    if (req.getParameter(CalParameter.START_MONTH).trim().equals("None")) {
        error_msg = "Start month is not selected!";
    } else if (req.getParameter(CalParameter.END_MONTH).trim().equals("None")) {
        error_msg = "End month is not selected!";
    }

        range_begin = pep.getStartYear() + sqlstmts.monthNumber(pep.getStartMonth()) + pep.getStartDay();
        range_end = pep.getEndYear() + sqlstmts.monthNumber(pep.getEndMonth()) + pep.getEndDay();


    if (error_msg == "") {
        try {
             rs = sqlstmts.executeQuery(
               new PubEventsQueries()
	           .adminUpViaDatesQuery(new MyCalendar(range_begin),
                                      new MyCalendar(range_end))
                   .toString());
        } catch (Exception e) {
            e.printStackTrace(System.err);
            System.err.println("Exception: " + e.getMessage());
            error_msg = "ERROR: " + e.getMessage();
        }

        try {
          if (error_msg == "") {
              if (!rs.next()) {
                 error_msg = "No record existed for this date range.";
              } else {
                 new PrintWebPage(templateUpdtEvents, out); // print out header
                 do {
                    out.println("<tr><td><input type=\"radio\" name=\"" + PEParameter.EVENT_ID + "\" value=\"" + rs.getString(Fields.EEVENT_ID.getName()) + "\"></td>");
                    out.println("<td>" + rs.getDate(Fields.ESTARTDATE.getName()) + "</td>");
                    out.println("<td>" + rs.getDate(Fields.EENDDATE.getName()) + "</td>");
                    out.println("<td>" + rs.getString(Fields.ESHORTDESC.getName()) + "</td>");
                    out.print("<td>" + Fields.getEventLongdesc(rs));
                    out.println("</td></tr>");
                 } while (rs.next());
                 out.println("<tr><td></td><td></td><td></td><td></td></tr>");
                 out.println("<tr><td></td><td></td>");
                 out.println("<td><input type=\"submit\" value=\"Submit\"></td>");
                 out.println("<td><input type=\"reset\" value=\"Clear\"></td>");
                 out.println("<td></td></tr>");
                 out.println("</table>\n</form>");
                 new PrintWebPage(templateFooter, out); // print out footer
              }
          }
        } catch (Exception e) {
            e.printStackTrace(System.err);
            System.err.println("Exception: " + e.getMessage());
            error_msg = "ERROR: " + e.getMessage();
        } finally {
          sqlstmts.close();
        }
    }

        // print out error_msg, such as no record existed for this date range. 
    if (error_msg != "") {
        smon = pep.getStartMonth().substring(0, 3);
        sday = pep.getStartDay();
        syear = pep.getStartYear();
        emon = pep.getEndMonth().substring(0, 3);
        eday = pep.getEndDay();
        eyear = pep.getEndYear();

        templateFile = templateUpdtViaDates;
        BufferedReader in = new BufferedReader(new InputStreamReader(
                this.getClass().getResourceAsStream(templateFile)));

        String aline = in.readLine(); // IOException
        while (aline != null) {
          if (aline.startsWith("%%%html_title")) {
              out.println(html_title);
          } else if (aline.startsWith("%%%error_msg")) {
              new PrintErrorMessage().printError(out, error_msg);
          } else if (aline.startsWith("%%%action")) {
              out.println("\"pubevents.updtevents.AdminUpViaDates\"");
          } else if (aline.startsWith("%%%" + CalParameter.FUNCTION)) {
              out.println("\"" + PEDefaults.UPDATE_FUNCTION + "\"");
          } else if (aline.startsWith("%%%sel_smon")) {
                     if (aline.indexOf(smon) != -1) {
                         out.println("selected");
                     }
          } else if (aline.startsWith("%%%sel_sday")) {
                     if (aline.indexOf(sday) != -1) {
                         out.println("selected");
                     }
          } else if (aline.startsWith("%%%sel_syear")) {
                     if (aline.indexOf(syear) != -1) {
                         out.println("selected");
                     }
          } else if (aline.startsWith("%%%sel_emon")) {
                     if (aline.indexOf(emon) != -1) {
                         out.println("selected");
                     }
          } else if (aline.startsWith("%%%sel_eday")) {
                     if (aline.indexOf(eday) != -1) {
                         out.println("selected");
                     }
          } else if (aline.startsWith("%%%sel_eyear")) {
                     if (aline.indexOf(eyear) != -1) {
                         out.println("selected");
                     }
          } else if (aline.startsWith("%%%")) {
              // skip this line
          } else {
              out.println(aline);
          }
          aline = in.readLine();
        }
        in.close();
    }
    out.close();
  }
}
