package pubevents.updtevents;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.db.DBResources;
import edu.washington.cac.calendar.db.Fields;
import edu.washington.cac.calendar.db.PubEventsQueries;
import edu.washington.cac.calendar.db.Tables;

import uwcal.out.WebResources;

import pubevents.AdminServlet;
import pubevents.PEDefaults;
import pubevents.PEParameter;
import pubevents.addevents.SqlStatements;
import pubevents.util.DataUtil;
import pubevents.util.PrintErrorMessage;
import pubevents.util.PrintWebPage;

/** Display an event in a form
  
  @author Greg Barnes, Leman Chung
  @version 1.1
 */
public class EventFormDisplayer 
{
  private String templateFile = PEDefaults.TO_BE_UPDATED_EVENTS;
  private int eventid, sponsorid, locationid, seq, keywordid;
  private java.sql.Date startdate, enddate;
  private java.sql.Time starttime, endtime;
  private String shortdesc, longdesc, url, cost, creator;
  private String sponsor, contactPhone, contactEmail, sponsorUrl;
  private String areaCode, phoneBegin, phoneEnd, morePhone;
  private String location, locationUrl;
  private String word, uid;
  private String smonth, sday, syear, shour, smin, sampm;
  private String emonth, eday, eyear, ehour, emin, eampm;
  private String redAlert;
  private String error_msg = "";
  private ResultSet rs = null;

  /** The name of the function the form represents */
  private String function;

  /** The template file of a message to display if something goes wrong */
  private String backToMenu;

  /** The title of the HTML page */
  private String title;

  /** The parameters from a previous form */
  private PEParameter pep;

  /** The stream to write the page to */
  private PrintWriter out;

  /** Initialize the object
    @param req HttpServletRequest object
    @param resp HttpServletResponse object
    @param function Function name for the form to generate
    @param backToMenu Template for page to show if something is horribly wrong
    @exception IOException if there's trouble getting the output stream
   */
  public EventFormDisplayer(HttpServletRequest req, 
                     HttpServletResponse resp,
                     String function,
                     String backToMenu,
                     String title)
      throws IOException
  {
           // called via administrator's function
    this(req, resp, function, backToMenu, title, null);
  }

  /** Initialize the object
    @param req HttpServletRequest object
    @param resp HttpServletResponse object
    @param function Function name for the form to generate
    @param backToMenu Template for page to show if something is horribly wrong
    @exception IOException if there's trouble getting the output stream
   */
  public EventFormDisplayer(HttpServletRequest req, 
                     HttpServletResponse resp,
                     String function,
                     String backToMenu,
                     String title,
                     String uid)
      throws IOException
  {
    this.function = function;
    this.backToMenu = backToMenu;
    this.title = title;
    this.pep = new PEParameter(req);
    this.out = resp.getWriter();
    this.uid = uid;

    resp.setContentType("text/html");
  }

  /** Initialize the object
    @param req HttpServletRequest object
    @param resp HttpServletResponse object
    @param function Function name for the form to generate
    @param backToMenu Template for page to show if something is horribly wrong
    @exception IOException if there's trouble getting the output stream
   */
  public EventFormDisplayer(HttpServletRequest req, 
                     HttpServletResponse resp,
                     String function,
                     String backToMenu,
                     String title,
                     String uid,
                     String templateFile)
      throws IOException
  {
    this.function = function;
    this.backToMenu = backToMenu;
    this.title = title;
    this.pep = new PEParameter(req);
    this.out = resp.getWriter();
    this.uid = uid;
    this.templateFile = templateFile;
    resp.setContentType("text/html");
  }

  /** Display the form with parameters filled in from the database 
    @param IOException If we have trouble writing to the output stream
      or reading from the template file
   */
  public void displayAlertFromDB() throws SQLException, IOException
  {

      eventid = pep.getEventID();

      if (eventid == edu.washington.cac.calendar.data.Cacheable.NO_ID) {
        out.println("<br><br><b>No alert has been selected to update.</b>");
        new PrintWebPage(this.backToMenu, out);
        out.close();
        return;
      }
  
      // select an alert from db table.
  
      SqlStatements sqlstmts = new SqlStatements ();
      String qry_string = "select " 
                        + Fields.EEVENT_ID.getSelectName() + ", "
                        + Fields.ESEQ.getSelectName() + ", "
                        + Fields.ESTARTDATE.getSelectName() + ", "
                        + Fields.ESTARTTIME.getSelectName() + ", "
                        + Fields.EENDDATE.getSelectName() + ", "
                        + Fields.EENDTIME.getSelectName() + ", "
                        + Fields.ESHORTDESC.getSelectName() + ", "
                        + Fields.ELONGDESC.getSelectName() + ", "
                        + Fields.ELONGDESC1.getSelectName() + ", "
                        + Fields.ELINK.getSelectName() + ", "
                        + Fields.ECREATOR.getSelectName() + " from "
                        + Tables.EVENTS.getName() + " where "
                        + Fields.EEVENT_ID.getStmtName() + "="
                        + eventid;

    try {
        rs = sqlstmts.executeQuery(qry_string);
    } catch (Exception e) {
        e.printStackTrace(System.err);
        System.err.println(e.getMessage());
        throw new SQLException(e.getMessage());
    }

    try {
      if (rs.next()) {
          setVariableValues(rs);
      } else {
         error_msg = qry_string + " -- Alert not found!";
      }
      rs.close();
    } catch (Exception e) {
      e.printStackTrace(System.err);
      error_msg = "ERROR: " + e.getMessage();
    }

    try {
      rs = sqlstmts.executeQuery("select * from "
                                 + Tables.REDALERTS.getName() + " where "
                                 + Fields.REVENT_ID.getStmtName() + "="
                                 + eventid);
    } catch (Exception e) {
      e.printStackTrace(System.err);
      error_msg = "ERROR: " + e.getMessage();
    }

    try {
      if (rs.next()) {
          redAlert = "checked";
      } else {
          redAlert = "";
      }
      rs.close();
    } catch (Exception e) {
      e.printStackTrace(System.err);
      error_msg = "ERROR: " + e.getMessage();
    }

      sqlstmts.close();

      if (error_msg != "") {
           // display error_msg and exit.
        new PrintErrorMessage().printError(out, error_msg);
        new PrintWebPage(this.backToMenu, out);
        out.close();
        return;
      }

      // display alert over here
    displayEventFields();
    out.close();
  }

  /** Display the form with parameters filled in from the database 
    @param IOException If we have trouble writing to the output stream
      or reading from the template file
   */
  public void displayFromDB() throws SQLException, IOException
  {

      eventid = pep.getEventID();

      if (eventid == edu.washington.cac.calendar.data.Cacheable.NO_ID) {
        out.println("<br><br><b>No event has been selected to update.</b>");
        new PrintWebPage(this.backToMenu, out);
        out.close();
        return;
      }
  
      // select event from db tables and then convert date and time and then
      // move it to the corresponding html fields for display.
  
      // access db tables
      SqlStatements sqlstmts = new SqlStatements ();
      String qry_string = "select " 
                          + Fields.ESTARTDATE.getSelectName() + ", " 
                          + Fields.EENDDATE.getSelectName() + ", "
                          + Fields.ESHORTDESC.getSelectName() + ", "
                          + Fields.ELONGDESC.getSelectName() + ", "
                          + Fields.ELONGDESC1.getSelectName() + ", " 
                          + Fields.ESTARTTIME.getSelectName() + ", "
                          + Fields.EENDTIME.getSelectName() + ", " 
                          + Fields.ELINK.getSelectName() + ", "
                          + Fields.ECREATOR.getSelectName() + ", "
                          + Fields.ECOST.getSelectName() + ", " 
                          + Fields.ESPONSOR_ID.getSelectName() + ", "
                          + Fields.ELOCATION_ID.getSelectName() + ", "
                          + Fields.ESEQ.getSelectName() + ", "
                          + Fields.EKKEYWORD_ID.getSelectName() + ", "
                          + Fields.KWORD.getSelectName() + ", " 
                          + Fields.SNAME.getSelectName() + ", " 
                          + Fields.SPHONE.getSelectName() + ", " 
                          + Fields.SLINK.getSelectName() + ", " 
                          + Fields.SEMAIL.getSelectName() + ", " 
                          + Fields.LADDRESS.getSelectName() + ", " 
                          + Fields.LLINK.getSelectName() 
                          + " from "
                          + Tables.EVENTS.getName() + ", "
                          + Tables.EVENTKEYWORDS.getName() + ", "
                          + Tables.KEYWORDS.getName() + ", "
                          + Tables.LOCATIONS.getName() + ", "
                          + Tables.SPONSORS.getName() + " "
                          + new PubEventsQueries().baseStandardWhere() + " and "
                          + Fields.EEVENT_ID.getStmtName() + "=" + eventid 
                          + " and ";

      if (uid != null) {
        qry_string  += Fields.ECREATOR.getStmtName() + "="
                       + "\'" + uid + "\'"
                       + " and ";
      }

      qry_string  += Fields.ELOCATION_ID.getStmtName() + "="
                     + Fields.LLOCATION_ID.getStmtName() + " and "
                     + Fields.ESPONSOR_ID.getStmtName() + "="
                     + Fields.SSPONSOR_ID.getStmtName() + " and "
                     + Fields.KKEYWORD_ID.getStmtName() + "="
                     + Fields.EKKEYWORD_ID.getStmtName() + " and " 
                     + Fields.EEVENT_ID.getStmtName() + "=" 
                     + Fields.EKEVENT_ID.getStmtName();
  
      try {
        rs = sqlstmts.executeQuery(qry_string);
        if (rs.next()) {
          setVariableValues(rs);
        } else {
          error_msg = qry_string + " -- event not found!";
        }
        rs.close();
      } catch (Exception e) {
        e.printStackTrace(System.err);
        error_msg = "ERROR: " + e.getMessage();
      }

      sqlstmts.close();

      if (error_msg != "") {
           // display error_msg and exit.
        new PrintErrorMessage().printError(out, error_msg);
        new PrintWebPage(this.backToMenu, out);
        out.close();
        return;
      }
   
    // display alert over here
    // display alert via templatefile.
      displayEventFields();
      out.close();
  }

  private void setVariableValues(ResultSet rs) throws SQLException
  {
    sponsorid = rs.getInt(Fields.ESPONSOR_ID.getName());
    locationid = rs.getInt(Fields.ELOCATION_ID.getName());
    seq = rs.getInt(Fields.ESEQ.getName());
    startdate = rs.getDate(Fields.ESTARTDATE.getName());
    starttime = rs.getTime(Fields.ESTARTTIME.getName());
    enddate = rs.getDate(Fields.EENDDATE.getName());
    endtime = rs.getTime(Fields.EENDTIME.getName());
    shortdesc = rs.getString(Fields.ESHORTDESC.getName());
    longdesc = Fields.getEventLongdesc(rs);
    url = rs.getString(Fields.ELINK.getName());
    creator =  rs.getString(Fields.ECREATOR.getName());
    cost = rs.getString(Fields.ECOST.getName());
    sponsor = rs.getString(Fields.SNAME.getName());
    contactEmail = rs.getString(Fields.SEMAIL.getName());
    sponsorUrl = rs.getString(Fields.SLINK.getName());
    location = rs.getString(Fields.LADDRESS.getName());
    locationUrl = rs.getString(Fields.LLINK.getName());
    keywordid = rs.getInt(Fields.EKKEYWORD_ID.getName());
    word = rs.getString(Fields.KWORD.getName());
          
    // pad so the substrings will always work
    contactPhone = rs.getString(Fields.SPHONE.getName()) + 
              "               ";
    areaCode = contactPhone.substring(0, 3);
    phoneBegin = contactPhone.substring(4, 7);
    phoneEnd = contactPhone.substring(8, 12);
    morePhone = contactPhone.substring(13);

    if (url == null) {
      url = "";
    }

    if (cost == null) {
      cost = "";
    }

    if (creator == null) {
      creator = "";
    }

    if (contactEmail == null) {
      contactEmail = "";
    }

    if (sponsorUrl == null) {
      sponsorUrl = "";
    }

    if (locationUrl == null) {
      locationUrl = "";
    }
  }

  private void displayEventFields() throws IOException
  {
      // split date and time 
   
      SplitDateTime sdt = new SplitDateTime(startdate, starttime);
      smonth = new MyCalendar(startdate).twoDigitMonth();
      sday   = sdt.getDay();
      syear  = sdt.getYear();
      shour  = sdt.getHour();
      smin   = sdt.getMin();
      sampm = sdt.getAmPm();
   
      sdt = new SplitDateTime(enddate, endtime);
      emonth = new MyCalendar(enddate).twoDigitMonth();
      eday   = sdt.getDay();
      eyear  = sdt.getYear();
      ehour  = sdt.getHour();
      emin   = sdt.getMin();
      eampm = sdt.getAmPm();

      // read and print out new Form
      BufferedReader in = new BufferedReader(new InputStreamReader(
              this.getClass().getResourceAsStream(templateFile)));

      try {
        String aline = in.readLine(); 
     
        while (aline != null) {
          if (aline.startsWith("%%%error_msg")) {
            out.println(error_msg);
          } else if (aline.startsWith("%%%" + PEDefaults.CATEGORY_HELP_URL)) {
            out.println(WebResources.categoryHelpUrl());
          } else if (aline.startsWith("%%%html_title")) {
            out.println(this.title);
          } else if (aline.startsWith("%%%reset")) {
            out.println("\"Reset (back to original)\"");
          } else if (aline.startsWith("%%%" + CalParameter.LONG_DESCRIPTION)) {
            out.print(longdesc);  // special case, use print not println.
          } else if (aline.startsWith("%%%")) {
            out.println(getFieldValue(aline.substring(3)));
          } else {
            out.println(aline);
          }
     
          aline = in.readLine();
        }
      } finally {
        in.close();
      }
  }

  private String getFieldValue(String field) 
  {
    if (field.equals(CalParameter.FUNCTION)) {
      return this.function;
    } else if (field.equals(CalParameter.SPONSOR_NAME)) {
      return ("\"" + sponsor + "\"");
    } else if (field.equals(PEParameter.SLINK)) {
      return ("\"" + sponsorUrl + "\"");
    } else if (field.equals(PEParameter.AREA_CODE)) {
      return ("\"" + areaCode + "\"");
    } else if (field.equals(PEParameter.PHONE_BEGIN)) {
      return ("\"" + phoneBegin + "\"");
    } else if (field.equals(PEParameter.PHONE_END)) {
      return ("\"" + phoneEnd + "\"");
    } else if (field.equals(PEParameter.MORE_PHONE)) {
      return ("\"" + morePhone + "\"");
    } else if (field.equals(CalParameter.EMAIL)) {
      return ("\"" + contactEmail + "\"");
    } else if (field.equals(CalParameter.START_MONTH)) {
      return ("\"" + smonth + "\">" + new MyCalendar(startdate).monthString());
    } else if (field.equals(CalParameter.START_DAY)) {
      return sday;
    } else if (field.equals(CalParameter.START_YEAR)) {
      return syear;
    } else if (field.equals(CalParameter.END_MONTH)) {
      return ("\"" + emonth + "\">" + new MyCalendar(enddate).monthString());
    } else if (field.equals(CalParameter.END_DAY)) {
      return eday;
    } else if (field.equals(CalParameter.END_YEAR)) {
      return eyear;
    } else if (field.equals(CalParameter.WORD)) {
      return "\"" + word + "\">" + 
             (DataUtil.isOfficialKeyword(keywordid) ? "" : "* ") +
             word;
    } else if (field.equals(CalParameter.SHORT_DESCRIPTION)) {
      return ("\"" + shortdesc + "\"");
    } else if (field.equals(CalParameter.ADDRESS)) {
      return ("\"" + location + "\"");
    } else if (field.equals(PEParameter.LLINK)) {
      return ("\"" + locationUrl + "\"");
    } else if (field.equals(CalParameter.START_HOUR)) {
      return shour;
    } else if (field.equals(CalParameter.START_MINUTE)) {
      return smin;
    } else if (field.equals(CalParameter.START_AMPM)) {
      return sampm;
    } else if (field.equals(CalParameter.END_HOUR)) {
      return ehour;
    } else if (field.equals(CalParameter.END_MINUTE)) {
      return emin;
    } else if (field.equals(CalParameter.END_AMPM)) {
      return eampm;
    } else if (field.equals(CalParameter.LINK)) {
      return ("\"" + url + "\"");
    } else if (field.equals(CalParameter.COST)) {
      return ("\"" + cost + "\"");
    } else if (field.equals(PEParameter.CREATOR)) {
      return ("\"" + creator + "\"");
    } else if (field.equals(PEParameter.RED_ALERT)) {
      return redAlert;
    } else if (field.equals(CalParameter.SPONSOR_ID)) {
      return String.valueOf(sponsorid);
    } else if (field.equals(CalParameter.LOCATION_ID)) {
      return String.valueOf(locationid);
    } else if (field.equals(CalParameter.EVENT_ID)) {
      return String.valueOf(eventid);
    } else if (field.equals(PEParameter.SEQ)) {
      return String.valueOf(seq);
    } else if (field.equals(CalParameter.KEYWORD_ID)) {
      return String.valueOf(keywordid);
    } else if (field.equals(PEParameter.HIDDEN_KEYWORD)) {
      return word;
    } else {
      return "";
    }
  }
}
