// java

package uwcal;

import edu.washington.cac.myuw.servlet.session.Profile;

import edu.washington.cac.calendar.data.CaldataException;
import edu.washington.cac.calendar.data.EventsI;
import edu.washington.cac.calendar.data.NoSuchItemException;
import edu.washington.cac.calendar.data.PersonalEvents;
import edu.washington.cac.calendar.data.PublicEvents;
import edu.washington.cac.calendar.data.User;
import edu.washington.cac.calendar.db.Caldata;
import edu.washington.cac.calendar.db.DBResources;
import edu.washington.cac.calendar.filter.Filter;
import edu.washington.cac.calendar.filter.FilteredEvents;
import edu.washington.cac.calendar.filter.PublicCalendars;

/** Encapsulates all the information about a user stored in the
    database.  Most data is stored in a cache in the current
    CalSession.

    @author Greg Barnes
    @version 1.1
 */
public class CalProfile extends Profile 
{
  /** Create a profile 
    @param session The session to use to store the profile
   */
  public CalProfile(CalSession session) 
  {
    super(session);
  }

  /** The default calendar for a guest */
  public static final Filter DEFAULT_FILTER = 
      PublicCalendars.defaultCalendar().getFilter();

  /** The key for the events object in the profile's cache */
  private static final String EVENTS = "EVENTS";

  /**
    get the cache of events for the user
    @returns The cache of events for the user
    @exception java.sql.SQLException if we have trouble loading something
      from the database
    @exception CaldataException if we try loading bad data
      from the database
   */
  public EventsI getEvents() 
    throws java.sql.SQLException, CaldataException
  {
    if (session.getValue(EVENTS) == null) {
      if (((CalSession) this.session).getUser().isGuest()) {
        return new FilteredEvents(DEFAULT_FILTER);
      } else {
        return new PersonalEvents(((CalSession) this.session).getUser(),
                          new Caldata(getLog()));
      }
    } else {
      return (EventsI) session.getValue(EVENTS);
    }
  }

  /**
    Store the cache of events in the profile
    @param e The cache of events
   */
  public void storeEvents(EventsI e)
  {
    session.putValue(EVENTS, e);
  }
}
