package uwcal;

import javax.servlet.http.HttpServletRequest;

import edu.washington.cac.calendar.CalLogFactory;
import edu.washington.cac.calendar.data.User;
import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.myuw.servlet.session.Profile;
import edu.washington.cac.myuw.servlet.session.UWSession;
import edu.washington.cac.myuw.servlet.ParameterException;

/** handles creating, deleting, verifying user sessions for UWCal
   @author Greg Barnes
   @version 1.1
 */
public class CalSession extends UWSession 
{
  /** Create a CalSession
    @param req The current HTTP request object
   */
  public CalSession(HttpServletRequest req) 
  {
    super(req, new CalLogFactory());
  }

  private static final String USEROBJECT_KEY = User.class.getName();

  public Profile getProfile() 
  {
    log.println("Don't use CalSession.getProfile()!");
    return getCalProfile();
  }

  public CalProfile getCalProfile() 
  {
    Profile p;

    try {
      p = super.getProfile();
    } catch (ClassCastException e) {   
      /* can happen if we come from another application in the same 
         zone */
      return new CalProfile(this);
    }

    if (p instanceof CalProfile) {
      return (CalProfile) p;
    } else {
      // must be newly generated; generate our own.
      return new CalProfile(this);
    }
  }

  /** 
    Get the User object associated with this session 
    @return the User object associated with this session 
   */
  public User getUser() 
  {
    return (User) getValue(USEROBJECT_KEY);
  }

  /** 
    set the User object associated with this session 
    @param user New user
   */
  void setUser(User user) 
  {
    user.setIdentifier(getIdentifier());
    putValue(USEROBJECT_KEY, user);
  }

  /** 
     does the identifier value in the http request match the value stored 
       in the session?
     @return does the identifier value in the http request match the value 
       stored in the session?
   */
  public boolean matchedIdentifier() 
  {
    try {
      CalParameter p = new CalParameter(this.req);
    return (p.isUserFunction() && !p.isAddEventRefFunction()) 
      ? matchedIdentifier(p.getIdentifier()) : true;
    } catch (ParameterException e ) {
      return false;
    }
  }
}
