// java

package uwcal;

import java.sql.SQLException;

import edu.washington.cac.myuw.servlet.Log;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.EventsI;
import edu.washington.cac.calendar.data.ItemException;
import edu.washington.cac.calendar.data.User;

public class DeleteAction extends DBAction {
  /** Id # of the object to delete */
  private int id;

  /** Create a minimal DeleteAction, not suitable for performing the
      action itself.  Used for matching function names to the appropriate
      Action object, and creating a full instance of the object
    @param function Name of the function this action can handle
    @param successMessage Message to display if this action succeeds
   */
  DeleteAction(String function, String successMessage) 
  {
    super(function, successMessage);
  }

  /** Construct a new DeleteAction that can perform the action

    @param function Name of the function to perform
    @param cal Calendar for the day in question
    @param user User performing the action
    @param es EventsI object for the user
    @param cp Form parameters
    @param log File to print messages to
    @param successMessage Message to print if the action succeeds
   */
  protected DeleteAction(String function,       
                         MyCalendar cal,
                         User user, 
                         EventsI es,
                         CalParameter cp,
                         Log log,
                         String successMessage)
  {
    super(function, cal, user, es, cp, log, successMessage);
  }

  /** @return a new DeleteAction

    @param cal Calendar for the day in question
    @param user User performing the action
    @param es EventsI object for the user
    @param cp Form parameters
    @param log File to print messages to
   */
  CalAction newInstance(MyCalendar cal, 
                        User user, 
                        EventsI es, 
                        CalParameter cp,
                        Log log) 
  {
    return new DeleteAction(getHandlesFunction(), cal, user, es, cp, log, 
                            getSuccessMessage());
  }

  /**
    Set the id of the object to delete
   */
  private void setId() 
  {
    if (getHandlesFunction().endsWith(CalParameter.KEYWORD)) {
      id = cp.getKeywordID();
    } else if (getHandlesFunction().endsWith(CalParameter.SPONSOR)) {
      id = cp.getSponsorID();
    } else if (getHandlesFunction().endsWith(CalParameter.LOCATION)) {
      id = cp.getLocationID();
    } else if (getHandlesFunction().endsWith(CalParameter.EVENT) || 
      getHandlesFunction().endsWith(CalParameter.FULLEVENT))
    {
      id = cp.getEventID();
    } else {
      throw new IllegalStateException("setId: illegalAction " + 
                                      getHandlesFunction());
    }
  }
  
  /**
    Perform the delete
    @exception SQLException if there's a database problem
    @exception ItemException if there's a problem with the item
   */
  protected void perfAction() throws SQLException, ItemException 
  {
    getCache().delete(getCache().get(id));
  }

  /**
    Set up the variables needed to perform the action
   */
  protected void setVars() 
  {
    setCache();
    setId();
  }
}
