/*++ExportPage.java+++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* Log:
 * see eof
 */


package  uwcal.export;


import  java.io.PrintWriter;
import  java.io.IOException;
import  java.util.GregorianCalendar;
import  javax.servlet.ServletConfig;
import  javax.servlet.ServletException;
import  javax.servlet.http.HttpServletRequest;
import  javax.servlet.http.HttpServletResponse;

import  edu.washington.cac.myuw.servlet.Log;
import  edu.washington.cac.myuw.servlet.BlankParameterException;
import  edu.washington.cac.myuw.servlet.NullParameterException;
import  edu.washington.cac.myuw.servlet.ServletSupport;
import  edu.washington.cac.myuw.servlet.session.UWSession;
import  edu.washington.cac.calendar.CalLogFactory;
import  edu.washington.cac.calendar.CalParameter;
import  edu.washington.cac.calendar.data.Events;
import  edu.washington.cac.calendar.data.EventsI;
import  edu.washington.cac.calendar.data.User;
import  edu.washington.cac.calendar.db.Caldata;
import  edu.washington.cac.calendar.icalendar.ICalendar;
import  edu.washington.cac.calendar.icalendar.Utilities;
import  edu.washington.cac.calendar.uwical.UWICal;

import  uwcal.CalServlet;
import  uwcal.CalSession;
import  uwcal.out.CalMessageMaker;


/**
 * Export user's calendar page.
 *
 * @author slh
 * @version 0.02 2002/02/27 slh
 */
public class  ExportPage  extends  CalServlet  {

/*----------------------------------------------------------------------------
 *                                              Object Public Methods
 *--------------------------------------------------------------------------*/

  public
  void  init  (ServletConfig    sc      )
  throws  ServletException
  {
    super.init( sc );

    String      strMethod       = "init";

    strModule = this.getClass(  ).getName(  );
    id = strModule;

    createLog().log( strModule , strMethod , Log.LOGTYPE_NOTICE ,
                     "Starting servlet..." );
  }


  public
  void  doGet  (HttpServletRequest      hsreq   ,
                HttpServletResponse     hsres   )
  throws  ServletException, IOException
  {
    String              strMethod       = "doGet";

    ServletSupport      ss;
    CalSession          cs;
    User user           = null;
    PrintWriter         pw;
    Log                 log;
    CalParameter        cp;
    String              strBegTime      = "";
    String              strEndTime      = "";
    boolean             bMime           = true;
    int                 Error           = mc_Error_NoError;

    ss = new ServletSupport( strModule ,
                             new CalMessageMaker(  ) , new CalLogFactory(  ) ,
                             true );

    try {
      cs = new CalSession( hsreq );
      ss.setLog( cs.getLog(  ) );
      log = cs.getLog(  );
      cs.verify(  );
      user = cs.getUser();

      cp = new CalParameter( hsreq );

      try {
        /*Note: get least error prone parameters first*/
        Error = Integer.parseInt( cp.getPValue( mc_strParamName_Error ) );
        bMime = null != cp.getPValue( mc_strParamName_Mime , false );
        strEndTime = cp.getPValue( mc_strParamName_End );
        strBegTime = cp.getPValue( mc_strParamName_Begin );
      } catch (Exception        e       ) {
        ;       // ignore any errors here
      }

      pw = hsres.getWriter(  );

      hsres.setContentType( mc_strContentType_html );

      pw.print( "<HTML>\n" +
                "<HEAD>\n" +
                "<TITLE>Calendar Export Form</TITLE>\n" +
                "</HEAD>\n" +
                "<BODY>\n" );

      if (user.isGuest()) {
        pw.print( "<STRONG>Public Calendar</STRONG>\n" );
      } else {
        pw.print( "<STRONG>" + user.getNameDB() + "'s Calendar</STRONG>\n" );
      }

      if (Error != mc_Error_NoError) {
        pw.print( "<P><STRONG>" );
        switch (Error) {
        case mc_Error_MissingData:
          pw.print( "One or more fields were not filled-in." );
          break;
        case mc_Error_InvalidData:
          pw.print( "One or more fields contained invalid data." );
          break;
        case mc_Error_NullParameter:
          pw.print( "Unknown error." );
          break;
        case mc_Error_NoError:
        default:
          break;
        }
        pw.println( "</STRONG>" );
      }

      pw.print( "<TABLE>\n" );
      pw.print( "<FORM ACTION='" + mc_strURL_Self + "'" +
                " METHOD=POST>\n" );
      genTextItem( pw , strBegTime ,
                   mc_strParamName_Begin , "b" , mc_strParamLabel_Begin ,
                   "Beginning date of time period for which to retrieve data" +
                   " (format: YYYYMMDD; e.g.: 20011001)." ,
                   true );
      genTextItem( pw , strEndTime ,
                   mc_strParamName_End , "e" , mc_strParamLabel_End ,
                   "Ending date of time period for which to retrieve data" +
                   " (format: YYYYMMDD; e.g.: 20011031)." ,
                   true );
      genCheckboxItem(
                pw , bMime ,
                mc_strParamName_Mime , "m" , mc_strParamLabel_Mime ,
                "MIME encapsulate including quoted-printable encoding?" ,
                true );
      pw.print( "<TR><TD>\n" );
      pw.print( "<INPUT TYPE='SUBMIT'" +
                " NAME='" + mc_strParamName_Retrieve + "'" +
                " VALUE='" + mc_strParamLabel_Retrieve + "'>\n" );
      pw.print( "</FORM>\n" );
      pw.print( "</TD></TR>\n" );
      pw.print( "</TABLE>\n" );

      pw.print( "</BODY>\n" +
                "</HTML>\n" );
    } catch (Throwable  e       ) {
      ss.handleException( e , hsres );
    } finally {
      ss.done(  );
    }
  }


  public
  void  doPost  (HttpServletRequest     hsreq   ,
                 HttpServletResponse    hsres   )
  throws  ServletException, IOException
  {
    String              strMethod       = "doPost";

    ServletSupport      ss;
    CalSession          cs;
    User user = null;
    EventsI              events;
    PrintWriter         pw;
    ICalendar           icalendar;
    Log                 log;
    CalParameter        cp;
    String      strBegTime      = null;
    String      strEndTime      = null;
    boolean     bMime           = false;
    int[]       aBeg            = new int[3];
    int[]       aEnd            = new int[3];
    int         Error           = mc_Error_NoError;

    ss = new ServletSupport( strModule ,
                             new CalMessageMaker(  ) , new CalLogFactory(  ) ,
                             true );

    try {
      cs = new CalSession( hsreq );
      ss.setLog( cs.getLog(  ) );
      log = cs.getLog(  );
      cs.verify(  );
      user = cs.getUser();

      cp = new CalParameter( hsreq );

      try {
        /*Note: get least error prone parameters first*/
        bMime = null != cp.getPValue( mc_strParamName_Mime , false );
        strEndTime = cp.getPValue( mc_strParamName_End );
        strBegTime = cp.getPValue( mc_strParamName_Begin );
      } catch (BlankParameterException  e       ) {
        Error = mc_Error_MissingData;
      } catch (NullParameterException   e       ) {
      /*NOTE: not clear when this happens;
         maybe when a user with an old browser on a slow connection
         cancels update
      */
        log.log( strModule , strMethod , Log.LOGTYPE_WARNING ,
                 "a page parameter is null: aborting" );
        Error = mc_Error_NullParameter;
      }

      if (Error != mc_Error_NoError) {
        redirect( hsreq , hsres , strBegTime , strEndTime , bMime , Error );
        return;                                                 /*RETURN*/
      }

      try {
        aBeg = Utilities.splitNumber( strBegTime , new int[] {4, 2, 2} );
        aEnd = Utilities.splitNumber( strEndTime , new int[] {4, 2, 2} );
        aBeg[1]--;      /* month is zero-based */
        aEnd[1]--;      /* month is zero-based */
      } catch (NumberFormatException    e       ) {
        Error = mc_Error_InvalidData;
      }

      if (Error != mc_Error_NoError) {
        redirect( hsreq , hsres , strBegTime , strEndTime , bMime , Error );
        return;                                                 /*RETURN*/
      }

      log.log( strModule , strMethod , Log.LOGTYPE_NOTICE ,
               "parameters: " + strBegTime + " " + strEndTime );

      pw = hsres.getWriter(  );

      if (user.isGuest()) {
        events = new Events( new Caldata(  ) );
      } else {
        events = new Events(user, new Caldata());
      }

      try {
        /*void*/events.prepareManyDays(
                new GregorianCalendar(
                        aBeg[0] , aBeg[1] , aBeg[2] , 0 , 0 , 0 ) ,
                new GregorianCalendar(
                        aEnd[0] , aEnd[1] , aEnd[2] , 0 , 0 , 0 ) );
      } catch (Exception        e       ) {
        hsres.setContentType( mc_strContentType_html );
        hsres.setHeader( mc_strHeaderName_Pragma ,
                         mc_strHeaderValue_NoCache );
        pw.print( "<HTML>\n" +
                  "<HEAD>\n" +
                  "<TITLE>Calendar Export Error</TITLE>\n" +
                  "</HEAD>\n" +
                  "<BODY>\n" );

        pw.print( "<PRE>\n" );
        pw.print( e.getLocalizedMessage(  ) + "\n" );
        pw.print( "</PRE>\n" );

        pw.print( "</BODY>\n" +
                  "</HTML>\n" );
//???this code is repeated in the catch below...
      }

      try {
        hsres.setHeader( mc_strHeaderName_Pragma ,
                         mc_strHeaderValue_NoCache );

        icalendar = new UWICal( "C&C/UWCal Export" , "2.0" , events );

        if (bMime) {
          hsres.setContentType( mc_strContentType_mime );
          pw.print( Utilities.encode( icalendar ) );
        } else {
          hsres.setContentType( mc_strContentType_calendar );
          pw.print( Utilities.foldLines( icalendar ) );
        }
      } catch (Exception        e       ) {
        hsres.setContentType( mc_strContentType_html );
        hsres.setHeader( mc_strHeaderName_Pragma ,
                         mc_strHeaderValue_NoCache );
        pw.print( "<HTML>\n" +
                  "<HEAD>\n" +
                  "<TITLE>Calendar Export Error</TITLE>\n" +
                  "</HEAD>\n" +
                  "<BODY>\n" );

        pw.print( "<PRE>\n" );
        pw.print( e.getLocalizedMessage(  ) + "\n" );
        pw.print( "</PRE>\n" );

        pw.print( "</BODY>\n" +
                  "</HTML>\n" );
      }
    } catch (Throwable  e       ) {
      ss.handleException( e , hsres );
    } finally {
      ss.done(  );
    }
  }


  public
  String  getServletInfo  (  )
  {
    return strModule + " servlet";
  }


/*----------------------------------------------------------------------------
 *                                              Class Private Methods
 *--------------------------------------------------------------------------*/

  static private
  void  redirect  (HttpServletRequest   hsreq           ,
                   HttpServletResponse  hsres           ,
                   String               strBegTime      ,
                   String               strEndTime      ,
                   boolean              bMime           ,
                   int                  Error           )
  throws Exception
  {
    /*void*/sendRedirect( hsreq , hsres ,
                          mc_strURL_Self + "?" +
                          mc_strParamName_Error + "=" + Error +
                          (!bMime ? "" : "&" + mc_strParamName_Mime + "=on") +
                          (strEndTime == null ? "" :
                           ("&" + mc_strParamName_End + "=" + strEndTime)) +
                          (strBegTime == null ? "" :
                           ("&" + mc_strParamName_Begin + "=" + strBegTime)) ,
                          true , true );
  }


  /**
   * GENerates (to pw) a TEXT ITEM.
   */
  static private
  void  genTextItem  (PrintWriter       pw              ,
                      String            strValue        ,
                      String            strParamName    ,
                      String            strAccessKey    ,
                      String            strLabel        ,
                      String            strDescr        ,
                      boolean           bTable          )
  throws IOException
  {
    if (bTable) {
      pw.println( "<TR><TD>" );
    }
    genLabelTag( pw , strParamName , strAccessKey , strLabel );
    if (bTable) {
      pw.println( "</TD><TD>" );
    }
    pw.println( "<INPUT TYPE='text'" +
                " NAME='" + strParamName + "'" +
                " ID='" + strParamName + "'" +
                " VALUE='" + strValue + "'>" );
    if (bTable) {
      pw.println( "</TD></TR>" );
    }
    genDescr( pw , strDescr , bTable );
  }


  /**
   * GENerates (to prw) CHECKBOX ITEM.
   */
  private static
  void  genCheckboxItem  (PrintWriter   prw             ,
                          boolean       bSelected       ,
                          String        strParamName    ,
                          String        strAccessKey    ,
                          String        strLabel        ,
                          String        strDescr        ,
                          boolean       bTable          )
  throws java.io.IOException
  {
    if (bTable) {
      prw.println( "<TR><TD>" );
    }
    genLabelTag( prw , strParamName , strAccessKey , strLabel );
    if (bTable) {
      prw.println( "</TD><TD>" );
    }
    prw.println( "<INPUT TYPE='checkbox'" +
                 " NAME='" + strParamName + "'" +
                 " ID='" + strParamName + "'" +
                 " VALUE='on'" +
                 (bSelected ? " CHECKED" : "") + ">" );
    if (bTable) {
      prw.println( "</TD></TR>" );
    }
    genDescr( prw , strDescr , bTable );
  }


  /**
   * GENerates (to pw) HTML for LABEL tag.
   */
  static private
  void  genLabelTag  (PrintWriter       pw              ,
                      String            strId           ,
                      String            strAccessKey    ,
                      String            strLabel        )
  throws IOException
  {
    pw.print( "<LABEL" );
    if (strId.equals( "" )) {
      pw.print( "FOR='" + strId + "'" );
    }
    if (strAccessKey.equals( "" )) {
      pw.print( " ACCESSKEY='" + strAccessKey + "'" );
    }
    pw.print( ">" + strLabel + " :</LABEL>" );
  }


  /**
   * GENerates (to pw) html for DESCRiptions for items.
   */
  private static
  void  genDescr  (PrintWriter  pw              ,
                   String       strText         ,
                   boolean      bTable          )
  throws IOException
  {
    if (bTable) {
      pw.println( "<TR><TD COLSPAN=2>" );
    }
    pw.println( "<SMALL>" + mc_strHTML_indent + strText + "</SMALL>" );
    if (bTable) {
      pw.println( "</TD></TR>" );
    }
  }


/*----------------------------------------------------------------------------
 *                                              Object Private Variables
 *--------------------------------------------------------------------------*/
  private String        strModule       = null;


/*----------------------------------------------------------------------------
 *                                              Class Private Constants
 *--------------------------------------------------------------------------*/
 /* Parameters and Values */
  static private final String   mc_strParamName_Begin           = "begin";
  static private final String   mc_strParamLabel_Begin          = "Begin Date";
  static private final String   mc_strParamName_End             = "end";
  static private final String   mc_strParamLabel_End            = "End Date";
  static private final String   mc_strParamName_Mime            = "mime";
  static private final String   mc_strParamLabel_Mime           =
                                                "MIME Encode?";
  static private final String   mc_strParamName_Error           = "error";
  static private final String   mc_strParamName_Retrieve        = "retrieve";
  static private final String   mc_strParamLabel_Retrieve       = "Retrieve";
 
  static private final int      mc_Error_NoError                = 0;
  static private final int      mc_Error_MissingData            = 1;
  static private final int      mc_Error_InvalidData            = 2;
  static private final int      mc_Error_NullParameter          = 3;

 /* Misc */
  static private final String   mc_strHTML_indent       =
                                                "&nbsp;&nbsp;&nbsp;&nbsp;";

  static private final String   mc_strURL_Self          =
                                                "uwcal.export.ExportPage";

 /* Standard Strings */
  static public final String    mc_strContentType_html          = "text/html";
  static public final String    mc_strContentType_calendar      =
                                                "text/calendar";
  static public final String    mc_strContentType_mime          = "text/mime";
  static public final String    mc_strHeaderName_Pragma         = "Pragma";
  static public final String    mc_strHeaderValue_NoCache       = "no-cache";


}


/* Note: update class header too */
/* Log:
 *  0.02  2002/02/27  slh
 *        include prodid and version calendar properties
 *  0.01  2002/02/25 - 2002/02/26  slh
 *        change error handling
 *        add mime option
 *        misc
 *        update for changes in class interfaces and locations
 *  0.00  2001/10/22 - 2001/10/23, 2001/11/06 - 2001/11/07, 2001/11/28  slh
 *        create
 */
/*--ExportPage.java---------------------------------------------------------*/
