// java

package uwcal.filter;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.data.NoSuchItemException;
import edu.washington.cac.calendar.filter.Filter;
import edu.washington.cac.calendar.filter.PublicCalendars;
import edu.washington.cac.calendar.filter.SearchFilter;

import uwcal.CalProfile;

/**
  Given the inputs of a 'select filters' form, generate
  the corresponding filter
  @author Greg Barnes
  @version 1.2
 */
public class FilterParser
{
  private Filter filter = null;

  /**
    Create a new instance
    @param cp CalParameter object containing form inputs
   */
  public FilterParser(CalParameter cp) 
  {
    if (cp.getSearchString() != null) {
      this.filter = new SearchFilter(cp.getSearchString());
    } else {
      try {
        this.filter = PublicCalendars.getPublicCalendars().
                      getCalendar(cp.getCalendar()).getFilter();
      } catch (NoSuchItemException e) {
        this.filter = CalProfile.DEFAULT_FILTER;
      } catch (NumberFormatException e) {
        this.filter = CalProfile.DEFAULT_FILTER;
      }
    }
  }

  /**
    Get the filter generated by the form
    @return the filter
   */
  public Filter getFilter() 
  {
    return this.filter;
  }
}
