// java

package uwcal.out;

import java.util.Date;

import org.apache.ecs.Element;
import org.apache.ecs.html.IMG;
import org.apache.ecs.html.Span;
import org.apache.ecs.html.Table;
import org.apache.ecs.html.TD;
import org.apache.ecs.html.TR;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.Event;
import edu.washington.cac.calendar.data.ItemException;
import edu.washington.cac.calendar.data.NoSuchItemException;
import edu.washington.cac.calendar.db.Caldata;

/**
  Used for printing the prev/this/next links at the top of a page.
  @author Greg Barnes
  @version 1.3
 */
public abstract class DateHeader 
{
  /** The current date */
  protected MyCalendar mc;

  /** A view function indicating what unit of time the prev/this/next links
   represent */
  private String func;

  /** An optional prefix to prepend to any URLs */
  private String urlPrefix = null;

  /**
    Create a new instance for use outside the main application.
    @param mc The current date
    @param urlPrefix A prefix to prepend to URLs.  The URLs normally
      used are relative, and thus are useful only in the same zone
      this application is running in.  Thus, if you wish to create
      a header in a different zone, you need to supply the absolute
      URL of the zone that runs the main application.
   */
  public DateHeader(MyCalendar mc, String func, String urlPrefix) 
  {
    this.mc = mc;
    this.func = func;
    this.urlPrefix = urlPrefix;
  }

  /**
    Create a new instance
    @param mc The current date
   */
  public DateHeader(MyCalendar mc, String func) 
  {
    this(mc, func, null);
  }

  /** Get the date of the 'previous' unit of time */
  protected abstract MyCalendar prevDate();

  /** Get the name of this unit of time */
  protected abstract String thisString();

  /** Get the date of the 'next' unit of time */
  protected abstract MyCalendar nextDate();

  /**
    @return A url that will yield a page of the appropriate unit of time
    @param mc date for the url
   */
  public String url(MyCalendar mc)
  {
    return Page.baseUrl(this.func, this.urlPrefix) + mc.getDateDigits();
  }

//  /**
//    @return A url that will yield a page of the appropriate unit of time
//    @param mc date for the url
//    @param prefix URL prefix to use instead of the default
//   */
//  public String url(MyCalendar mc, String prefix)
//  {
//    return Page.baseUrl(this.func, prefix) + mc.getDateDigits();
//  }

  private Element prevNextCell(MyCalendar mc, 
                               String imageUrl, 
                               String alt,
                               int width,
                               int height,
                               String imageStyle,
                               String cellStyle)
  {
    return new TD().
           addElement(Util.linkItem(url(mc),
               new IMG(imageUrl).setAlt(alt).
setBorder(0).setWidth(width).setHeight(height). //%%% for non-compliant Browsers
               setID(imageStyle))).
           setID(cellStyle);
  }

  private Element prevCell() 
  {
    return prevNextCell(prevDate(), WebResources.PREV_URL, 
                        DisplayText.PREVIOUS, Style.PREV_WIDTH, 
                        Style.PREV_HEIGHT, Style.PREV_IMAGE, Style.PREV_CELL);
  }

  private Element textCell() 
  {
    return new TD().  //setNoWrap(true).
           addElement(Util.linkItem(url(this.mc), 
                                    new Span().addElement(thisString()).
                      setID(Style.DATE_HEADER_TEXT))).
           setID(Style.DATE_HEADER_TEXT_CELL);
  }

  private Element nextCell() 
  {
    return prevNextCell(nextDate(), WebResources.NEXT_URL, 
                        DisplayText.NEXT, Style.NEXT_WIDTH, Style.NEXT_HEIGHT,
                        Style.NEXT_IMAGE, Style.NEXT_CELL);
  }

  public Element getHeader() 
  {
    return new TD().
           addElement(new Table().setCellSpacing(0).setCellPadding(0).
                      addElement(new TR().
                                 addElement(prevCell()).
                                 addElement("\n").
                                 addElement(textCell()).
                                 addElement("\n").
                                 addElement(nextCell()).
                                 addElement("\n"))).
           setID(Style.DATE_HEADER_CELL);
  }
}
