// java

package uwcal.out;

import java.sql.SQLException;
import java.util.Date;

import org.apache.ecs.Element;
import org.apache.ecs.ElementContainer;
import org.apache.ecs.html.Span;
import org.apache.ecs.html.Table;
import org.apache.ecs.html.TD;
import org.apache.ecs.html.TH;
import org.apache.ecs.html.THead;
import org.apache.ecs.html.TR;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.CaldataException;
import edu.washington.cac.calendar.data.EventsI;
import edu.washington.cac.calendar.data.User;
import edu.washington.cac.myuw.servlet.session.UWSession;

/**
  A Dayview webpage
  @author Greg Barnes
  @version 2.1
 */
public class Dayview extends Timeview 
{
  /** The dayview function name */
  private static final String FUNC = CalParameter.DAYVIEW_FUNC;

  /**
    Create a new instance
    @param mc Today
    @param user User of the application
    @param es User's events cache
    @param message Message to display at the top of the page
   */
  protected Dayview(MyCalendar mc, User user, EventsI es, String message) 
  {
    super(mc, user, es, message, FUNC, mc, mc);
  }

  /**
    @return A url that yields a Dayview
    @param mc Date of the page
   */
  public static String url(MyCalendar mc) 
  {
    return url(mc, null);
  }

  /**
    @return A url that yields a Dayview
    @param mc Date of the page
    @param prefix Prefix to use for the URL instead of the default
   */
  public static String url(MyCalendar mc, String prefix) 
  {
    return (baseUrl(FUNC, prefix) + mc.getDateDigits());
  }

  /**
    Get the object that can print the prev/this/next links
    @return the object that can print the prev/this/next links
   */
  protected DateHeader getDateHeader() 
  {
    return new DayDateHeader(this.mc);
  }

  protected boolean isDayview() {
      return true;
  }

  protected boolean isToday() {
      return this.mc.isSameDay(new MyCalendar());
  }

  private static final int COLUMNS_FOR_ONE_EVENT = 3;

  /**
    @return the body of this page
    @exception SQLException if there's a problem loading the events
    @exception CaldataException if the events we try to load have bad data
   */
  protected Element contents() throws SQLException, CaldataException
  {
    return new EventListAndFormPrinter(this.mc, this.user, this.es,
        new HourlyEventList(this.user, this.mc, this),
        COLUMNS_FOR_ONE_EVENT,
        DisplayText.DAY_EVENTS_HEADER + this.mc.dateString(),
        this).contents();
  }

  protected String title() {
    return DisplayText.DAYVIEW_TITLE;
  }

  /**
    Should this page display the filter message?
    @return Should this view display the filter message?
   */
  protected boolean shouldShowFilterMessage()
  {
    return true;
  }
}
