// java

package uwcal.out;

import java.text.DateFormat;

import org.apache.ecs.AlignType;
import org.apache.ecs.Element;
import org.apache.ecs.html.Table;
import org.apache.ecs.html.TD;
import org.apache.ecs.html.TH;
import org.apache.ecs.html.TR;

import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.EventsI;

/** A GridPrinter whose cells contain lists of events
  @author Greg Barnes
  @version 1.0
 */
public abstract class GridEventPrinter extends GridPrinter 
{
    EventList el;
    MyCalendar now = new MyCalendar();

    protected GridEventPrinter(MyCalendar mc, EventsI es, EventList el) {
        super(mc, es);
        this.el = el;
    }

    private String dayClass(int weekpos) {
        return MyCalendar.isWeekendDay(weekpos) ? 
          Style.WEEKEND : Style.WEEKDAY;
    }

    static final String DATE_WIDTH = 
      100 / new MyCalendar().daysInWeek() + "%";

    protected Element dayOfWeekNameHeading(int weekpos) {
        return new TH().setWidth(DATE_WIDTH).
          addElement(mc.dayOfWeekName(weekpos)).setClass(dayClass(weekpos));
    }

    protected TR emptyDayNames() {
        return new TR().setAlign(AlignType.CENTER);
    }

    protected TD dateSquareTD(MyCalendar date) {
        TD ds = new TD().setWidth(DATE_WIDTH).setVAlign(AlignType.TOP);

        if (date.isSameDay(mc)) {
            ds.setID(Style.CURRENT_DAY);
        } else if (date.isSameDay(now)) {
            ds.setID(Style.TODAY);
        } else if (date.isWeekendDay()) {
            ds.setClass(Style.WEEKEND_BACK);
        } else {
            ds.setClass(Style.WEEKDAY_BACK);
        }

        return ds;
    }

    private TR dateRow(MyCalendar date) {
        return new TR().addElement(new TD().setAlign(AlignType.RIGHT).
          addElement(Util.linkItem(Dayview.url(date), 
            date.getComponent(DateFormat.DATE_FIELD), Style.BOLD_LINK_ITEM)));
    }
   
  
  /**
    get the contents of a square in the grid
    @param date The date of the square
    @return the contents as HTML
    @exception java.sql.SQLException if there's a problem getting the events
      in the square
    @exception edu.washington.cac.calendar.data.CaldataException if the events hold bad data
   */
  protected Element dateSquareContents(MyCalendar date) 
    throws java.sql.SQLException, edu.washington.cac.calendar.data.CaldataException
  {
    return new Table().setWidth("100%").addElement(dateRow(date)).
      addElement(el.eventsRow(date, es));
  }
}
