// java

package uwcal.out;

import java.sql.SQLException;

import org.apache.ecs.Element;
import org.apache.ecs.html.TD;
import org.apache.ecs.html.TR;

import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.CaldataException;
import edu.washington.cac.calendar.data.EventsI;

/** An object that creates a grid of days (like a week, or a month)
    @author Greg Barnes
    @version 1.0
  */
public abstract class GridPrinter 
{
  /** A <code>MyCalendar</code> representing a day in the grid */
  protected MyCalendar mc;

  /** A user's events */
  EventsI es;

    protected GridPrinter(MyCalendar mc, EventsI es) {
        this.mc = mc;
        this.es = es;
    }

    protected abstract TR emptyDayNames();
    protected abstract Element dayOfWeekNameHeading(int weekpos);

  /**
    Get the contents of a square in the grid
    @param date The date of the square
    @return The contents, as HTML
    @exception SQLException if the contents include events, and
      there's a problem loading the events
    @exception CaldataException if the contents include events, and
      the events contain bad data
   */
  protected abstract Element dateSquareContents(MyCalendar date)
    throws SQLException, CaldataException;

    protected abstract TD dateSquareTD(MyCalendar date);

  /** 'print' the grid 
    @return the grid, as an HTML element
    @exception SQLException if the grid contains events, and
      there is a problem loading them.
    @exception CaldataException if we try to load events, and they
      contain bad data
   */
  protected abstract Element print() throws SQLException, CaldataException;

    protected TR dayNames() {
        TR names = emptyDayNames();
        int i, weekpos = mc.getFirstDayOfWeek();

        for (i = 0; i < mc.daysInWeek(); i++)
        {
            names.addElement(dayOfWeekNameHeading(weekpos)).addElement("\n");
            weekpos = (weekpos + 1) % mc.daysInWeek();    
        }

        return names;
    }

  /** get a square in the grid 
    @return the square, as a table cell
    @exception SQLException if the square contains events, and
      there is a problem loading them.
    @exception CaldataException if we try to load events, and they
      contain bad data
   */
  protected TD dateSquare(MyCalendar date) 
    throws SQLException, CaldataException
  {
    return dateSquareTD(date).addElement(dateSquareContents(date));
  }
}
