// java

package uwcal.out;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.EventsI;
import edu.washington.cac.calendar.data.User;

/**
  A page that must remember what view it came from
  @author Greg Barnes
  @version 1.0
 */
public abstract class IntermediatePage extends Page
{
  /** The view it came from */
  protected Timeview view;

  /**
    Create a new instance
    @param es User's events cache
    @param message Message to display at the top of the page
    @param view View we came from
   */
  protected IntermediatePage(EventsI es, 
                             String message,
                             Timeview view) 
  {
    super(view.getMc(), view.getUser(), es, message);
    this.view = view;
  }

  // methods that allow us to properly select the active 
  // today/day/week/month/year buttons

  /**
    Did we come from the 'today' view 
    @return Did we come from the 'today' view 
   */
  protected boolean isToday() 
  {
    return this.view.isToday();
  }

  /**
    Did we come from the day view 
    @return Did we come from the day view 
   */
  protected boolean isDayview() 
  {
    return this.view.isDayview();
  }

  /**
    Did we come from the week view 
    @return Did we come from the week view 
   */
  protected boolean isWeekview() 
  {
    return this.view.isWeekview();
  }

  /**
    Did we come from the month view 
    @return Did we come from the month view 
   */
  protected boolean isMonthview() 
  {
    return this.view.isMonthview();
  }

  /**
    Did we come from the year view.  Currently this isn't possible, but
    what the heck.
    @return Did we come from the year view 
   */
  protected boolean isYearview() 
  {
    return this.view.isYearview();
  }

  /**
    Get the object that can print the prev/this/next links
    @return the object that can print the prev/this/next links
   */
  protected DateHeader getDateHeader() throws IllegalStateException
  {
    return this.view.getDateHeader();
  }

  /**
    Get the function name for the page you see most often when you follow
    a link from this page
    @return the function name for the page you see most often when you follow
    a link from this page
   */
  public String nextFunction()
  {
    return this.view.nextFunction();
  }
}
