// java

package uwcal.out;

import java.sql.SQLException;
import java.util.Calendar;

import org.apache.ecs.Element;
import org.apache.ecs.ElementContainer;
import org.apache.ecs.html.BR;

import edu.washington.cac.calendar.CalLogFactory;
import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.CaldataException;
import edu.washington.cac.calendar.data.EventsI;
import edu.washington.cac.calendar.data.User;

/**
  A page that prints a month
  @author Greg Barnes
  @version 1.3
 */
public class Monthview extends Timeview 
{
  /** The name of the monthview function */
  private static final String FUNC = CalParameter.MONTHVIEW_FUNC;

  /** Show the month as a grid?  (if false, show as a list) */
  private boolean showGrid;

  /** Create a new instance
    @param mc Current day
    @param user User of the app
    @param es User's events cache
    @param message Message to print at the top of the page
    @param showGrid Show the month as a grid?
   */
  protected Monthview(MyCalendar mc, 
                      User user, 
                      EventsI es, 
                      String message,
                      boolean showGrid) 
  {
    super(mc, user, es, message, FUNC,
          new MyCalendar(mc.firstDayOfThisMonth()), 
          new MyCalendar(mc.lastDayOfThisMonth()));
    this.showGrid = showGrid;
  }

  /**
    @return A url that will yield a monthview 
    @param mc Current date for the page
   */
  public static String url(MyCalendar mc) 
  {
    return url(mc, null);
  }

  /**
    @return A url that will yield a monthview 
    @param mc Current date for the page
    @param prefix URL prefix to use instead of the default
   */
  public static String url(MyCalendar mc, String prefix) 
  {
    return (baseUrl(FUNC, prefix) + mc.getDateDigits());
  }

  /**
    Get the object that can print the prev/this/next links
    @return the object that can print the prev/this/next links
   */
  protected DateHeader getDateHeader() 
  {
    return new MonthDateHeader(this.mc);
  }

  /**
    Is this a month view?
    @return true
   */
  protected boolean isMonthview() 
  {
    return true;
  }

  /**
    Get the header for the list of events
    @return the header for the list of events
   */
  private String headerString()
  {
    return DisplayText.DAY_EVENTS_HEADER + getFirstDay().dateString() + " - " +
           getLastDay().dateString();
  }

  /**
    @return the contents of the page as HTML
    @exception SQLException if there's a problem getting the events
    @exception CaldataException if the database contains bad data
   */
  protected Element contents() throws SQLException, CaldataException
  {
    this.es.prepareManyDays(getFirstDay().getCalendar(), 
                            getLastDay().getCalendar());

    if (this.showGrid) {
      return new ElementContainer().addElement(new BR()).addElement("\n").
             addElement(new BigMonthPrinter(this.mc, this.user, this.es, this).
                        print());
    } else {
      return 
          new EventListAndFormPrinter(this.mc, this.user, this.es, 
              new MultidayEventList(new MultidayEventPrinter(this.user, this.mc,
                                                             this)),
              1, headerString(), this).contents();
    }
  }

  protected String title() {
    return DisplayText.MONTHVIEW_TITLE;
  }

  /**
    Should this page display the filter message?
    @return Should this view display the filter message?
   */
  protected boolean shouldShowFilterMessage()
  {
    return true;
  }
}
