// java

package uwcal.schedule;

import java.io.IOException;
import java.sql.SQLException;
import java.util.Iterator;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import edu.washington.cac.calendar.CalLogFactory;
import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.data.CaldataException;
import edu.washington.cac.calendar.data.PersonalEvents;
import edu.washington.cac.calendar.data.NoSuchItemException;
import edu.washington.cac.calendar.data.User;

import uwcal.CalSession;
import uwcal.userlogin.CalLogin;

/**
  Add a series of public events to a user's personal calendar
  @author Greg Barnes
  @version 1.0
 */
public class AddSchedule extends CalLogin
{
  /** The name of the servlet */
  private static final String NAME = "AddSchedule";

  /** 
    Get the name of the servlet
    @return the name of the servlet
   */
  protected String getName() 
  {
    return NAME;
  }

  /**
    Get info about the servlet
    @return info about the servlet
   */
  public String getServletInfo() 
  {
    return "AddSchedule servlet: add a person's schedule to their personal calendar";
  }

  /**
    Send Http response
    @param req Http request object
    @param resp Http response object
    @param user Name of the user
    @param session The current session
    @exception IOException If there's trouble writing the response
    @exception SQLException If there's trouble getting the schedule from the db
    @exception CaldataException If the items in the schedule are bogus
    @exception NoSuchItemException If the items in the schedule reference 
      other, non-existent items
   */
  protected void sendResponse(HttpServletRequest req,
                              HttpServletResponse resp,
                              User user,
                              CalSession session)
      throws IOException, SQLException, CaldataException, NoSuchItemException
  {
    CalParameter cp = new CalParameter(req);
    PersonalEvents es = (PersonalEvents) session.getCalProfile().getEvents();
    es.removeSchedule(cp.getSchedule(), cp.getQuarter());
    es.addSchedule(cp.getScheduleIds().iterator(), cp.getSchedule());
    session.getCalProfile().storeEvents(es);
    new AddSchedulePage(user, cp.getSchedule()).print(resp);
  }
}
